/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse zum Verwalten der SeitenStreifenFreigabeEigenschaften.<p>
 * 
 * Folgende Werte werden in dieser Klasse gespeichert:<p>
 * 
 * - Status der Freigabe (aktiv/inaktiv)<br>
 * - Liste der Fahrstreifen<br>
 * - Liste der Schaltgrnde<p>
 *
 * ber die Methode init() knnen Default-Werte gesetzt werden.<br>
 * Der Zugriff auf die Inhalte erfolgt mit entsprechende Getter und Setter.
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class SeitenStreifenFreigabeEigenschaften
{
	/**
	 * Status der Seitenstreifenfreigabe. 
	 */
	private String m_status;	
	/**
	 * Liste der Fahrstreifen.
	 */
	private List<SystemObject> m_fahrstreifen = new ArrayList<SystemObject>();	
	/**
	 * Liste der Schaltgrnde.
	 */
	private List<String> m_schaltgruende = new ArrayList<String>();
	
	
	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public SeitenStreifenFreigabeEigenschaften()
	{
	}
	
	/**
	 * Initialisierung der SeitenStreifenFreigabeEigenschaften auf Defaultwerte.
	 */
	public void init()
	{
		m_status = null;
		m_fahrstreifen.clear();
		m_schaltgruende.clear();
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		StringBuffer string = new StringBuffer();
		
		string.append("SeitenStreifenFreigabeEigenschaften " +
				          " Status " +  m_status +
				          " Fahrstreifen " + m_fahrstreifen +
				          " Schaltgrnde " + m_schaltgruende);
		
		return string.toString();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj)
			return true;
		
		SeitenStreifenFreigabeEigenschaften sfe = (SeitenStreifenFreigabeEigenschaften)obj;
		if (sfe != null)
		{
			if (this.m_fahrstreifen.size() != sfe.getFahrstreifen().size())
				return false;
			
			for (int i = 0; i < m_fahrstreifen.size(); i++)
			{
				if (!m_fahrstreifen.get(i).equals(sfe.getFahrstreifen().get(i)))
					return false;
			}

			if (this.m_schaltgruende.size() != sfe.getSchaltgruende().size())
				return false;
			
			for (int i = 0; i < m_schaltgruende.size(); i++)
			{
				if (!m_schaltgruende.get(i).equals(sfe.getSchaltgruende().get(i)))
					return false;
			}

			if (!this.m_status.equals(sfe.getStatus()))
				return false;
			
			return true;
		}
				
		return false;
	}

	/**
	 * Gibt den Status der Seitenstreifenfreigabe zurck.
	 * 
	 * @return Status der Seitenstreifenfreigabe.
	 */
	public String getStatus()
	{
		return m_status;
	}

	/**
	 * Setzt die Variable m_status.
	 * @param status Wert auf den die Variable m_status gesetzt werden soll.
	 */
	public void setStatus(String status)
	{
		m_status = status;
	}

	/**
	 * Gibt die Liste der Fahrstreifen zurck.
	 * @return Liste der Fahrstreifen
	 */
	public List<SystemObject> getFahrstreifen()
	{
		return m_fahrstreifen;
	}

	/**
	 * Setzt die Variable m_fahrstreifen.
	 * @param fahrstreifen Wert auf den die Variable m_fahrstreifen gesetzt werden soll.
	 */
	public void setFahrstreifen(List<SystemObject> fahrstreifen)
	{
		m_fahrstreifen = fahrstreifen;
	}

	/**
	 * Fgt der Liste Fahrstreifen ein Element am Ende hinzu.
	 * 
	 * @param fahrstreifen Objekt Fahrstreifen.
	 */
	public void addFahrstreifen(SystemObject fahrstreifen)
	{
		if (m_fahrstreifen == null)
			m_fahrstreifen = new ArrayList<SystemObject>();
		
		m_fahrstreifen.add(fahrstreifen);
	}

	/**
	 * Gibt die Liste der Schaltgrnde zurck.
	 * @return Liste der Schaltgrnde
	 */
	public List<String> getSchaltgruende()
	{
		return m_schaltgruende;
	}

	/**
	 * Setzt die Variable m_schaltgruende.
	 * @param schaltgruende Wert auf den die Variable m_schaltgruende gesetzt werden soll.
	 */
	public void setSchaltgruende(List<String> schaltgruende)
	{
		m_schaltgruende = schaltgruende;
	}

	/**
	 * Fgt der Liste Schaltgrnde ein Element am Ende hinzu.
	 * 
	 * @param schaltgruende Objekt Schaltgrund.
	 */
	public void setSchaltgruende(String schaltgruende)
	{
		if (m_schaltgruende == null)
			m_schaltgruende = new ArrayList<String>();
		
		m_schaltgruende.add(schaltgruende);
	}
}
