/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.stau;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.DataModel;

/**
 * Diese Klasse speichert die Daten der Attributgruppe !atg.sswSbaUeStauGeschw!.<p>
 * 
 * ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.3 $ / $Date: 2013/02/14 07:32:49 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class ParameterStauGeschw
{
	/**
	 * Pid der Attributgruppe.
	 */
	private String m_pidAtg = "atg.sswSbaUeStauGeschw";
	/**
	 * Grenzwert Gte.
	 */
	private float m_guete;
	/**
	 * Parameterversorgung fr 2 Fahrstreifen.
	 */
	private ParameterStaugeschwFahrstreifen m_fahrstreifen2;

	/**
	 * Parameterversorgung fr 3 Fahrstreifen.
	 */
	private ParameterStaugeschwFahrstreifen m_fahrstreifen3;

	/**
	 * Parameterversorgung fr 4 Fahrstreifen.
	 */
	private ParameterStaugeschwFahrstreifen m_fahrstreifen4;

	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public ParameterStauGeschw()
	{
	}
	/**
	 * Konstruktor mit bernahme des bergebenen Datensatzes in die interne Struktur.
	 *  
	 * @param rdata Datensatz der Parameter.
	 */
	public ParameterStauGeschw(ResultData rdata)
	{
		setdaten(rdata);
	}
	
	/**
	 * bernahme des bergebenen Datensatzes in die interne Struktur.
	 * 
	 * @param rdata Datensatz der Parameter.
	 */
	public void setdaten(ResultData rdata)
	{
		if (rdata != null)
		{
			Data data = rdata.getData();
			if (data != null)
			{
				m_guete = data.getScaledValue("Gte").floatValue();
				
				if (m_fahrstreifen2 == null)
					m_fahrstreifen2 = new ParameterStaugeschwFahrstreifen();
							
				m_fahrstreifen2.setVStauEin(data.getItem("2Fahrstreifen").getUnscaledValue("VStauEin").intValue());
				m_fahrstreifen2.setVStauAus(data.getItem("2Fahrstreifen").getUnscaledValue("VStauAus").intValue());
				m_fahrstreifen2.setVDiffStau(data.getItem("2Fahrstreifen").getUnscaledValue("VDiffStau").intValue());
				m_fahrstreifen2.setQKfzStau(data.getItem("2Fahrstreifen").getUnscaledValue("QKfzStau").intValue());

				if (m_fahrstreifen3 == null)
					m_fahrstreifen3 = new ParameterStaugeschwFahrstreifen();
							
				m_fahrstreifen3.setVStauEin(data.getItem("3Fahrstreifen").getUnscaledValue("VStauEin").intValue());
				m_fahrstreifen3.setVStauAus(data.getItem("3Fahrstreifen").getUnscaledValue("VStauAus").intValue());
				m_fahrstreifen3.setVDiffStau(data.getItem("3Fahrstreifen").getUnscaledValue("VDiffStau").intValue());
				m_fahrstreifen3.setQKfzStau(data.getItem("3Fahrstreifen").getUnscaledValue("QKfzStau").intValue());

				if (m_fahrstreifen4 == null)
					m_fahrstreifen4 = new ParameterStaugeschwFahrstreifen();
							
				m_fahrstreifen4.setVStauEin(data.getItem("4Fahrstreifen").getUnscaledValue("VStauEin").intValue());
				m_fahrstreifen4.setVStauAus(data.getItem("4Fahrstreifen").getUnscaledValue("VStauAus").intValue());
				m_fahrstreifen4.setVDiffStau(data.getItem("4Fahrstreifen").getUnscaledValue("VDiffStau").intValue());
				m_fahrstreifen4.setQKfzStau(data.getItem("4Fahrstreifen").getUnscaledValue("QKfzStau").intValue());				
			}
			else
			{
				m_fahrstreifen2 = new ParameterStaugeschwFahrstreifen();
				m_fahrstreifen3 = new ParameterStaugeschwFahrstreifen();
				m_fahrstreifen4 = new ParameterStaugeschwFahrstreifen();
			}
		}		
	}

	/**
	 * Erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeHarmoGeschwDiff</code> mit
	 * den Daten der internen Struktur.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @return erzeugter Datensatz.
	 */
	public Data erzeugeDatensatz(ClientDavConnection connection)
	{
		DataModel dm = connection.getDataModel();		
		Data dataNeu = connection.createData(dm.getAttributeGroup(m_pidAtg));

		if (dataNeu != null)
		{	
			dataNeu.getScaledValue("Gte").set(m_guete);
			
			dataNeu.getItem("2Fahrstreifen").getUnscaledValue("VStauEin").set(m_fahrstreifen2.getVStauEin());
			dataNeu.getItem("2Fahrstreifen").getUnscaledValue("VStauAus").set(m_fahrstreifen2.getVStauAus());
			dataNeu.getItem("2Fahrstreifen").getScaledValue("VDiffStau").set(m_fahrstreifen2.getVDiffStau());
			dataNeu.getItem("2Fahrstreifen").getScaledValue("QKfzStau").set(m_fahrstreifen2.getQKfzStau());

			dataNeu.getItem("3Fahrstreifen").getUnscaledValue("VStauEin").set(m_fahrstreifen3.getVStauEin());
			dataNeu.getItem("3Fahrstreifen").getUnscaledValue("VStauAus").set(m_fahrstreifen3.getVStauAus());
			dataNeu.getItem("3Fahrstreifen").getScaledValue("VDiffStau").set(m_fahrstreifen3.getVDiffStau());
			dataNeu.getItem("3Fahrstreifen").getScaledValue("QKfzStau").set(m_fahrstreifen3.getQKfzStau());

			dataNeu.getItem("4Fahrstreifen").getUnscaledValue("VStauEin").set(m_fahrstreifen4.getVStauEin());
			dataNeu.getItem("4Fahrstreifen").getUnscaledValue("VStauAus").set(m_fahrstreifen4.getVStauAus());
			dataNeu.getItem("4Fahrstreifen").getScaledValue("VDiffStau").set(m_fahrstreifen4.getVDiffStau());
			dataNeu.getItem("4Fahrstreifen").getScaledValue("QKfzStau").set(m_fahrstreifen4.getQKfzStau());

			return dataNeu;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() 
	{
		StringBuffer string = new StringBuffer();
		
		string.append("Gte " + m_guete +					
				          "\n 2 Fahrstreifen " + m_fahrstreifen2 +
				          "\n 3 Fahrstreifen " + m_fahrstreifen3 +
				          "\n 4 Fahrstreifen " + m_fahrstreifen4);
		
		return string.toString();
  }

	/**
	 * Gibt fr die bergebene Anzahl Fahrstreifen den entsprechenden Datensatz zurck.
	 * 
	 * @param anzFs Anzahl der Fahrstreifen.
	 * @return Parameterversorgung der bergebenen Anzahl von Fahrstreifen.
	 */
	public ParameterStaugeschwFahrstreifen getParameterFahrstreifen(int anzFs)
	{
		switch (anzFs)
		{
			case 2:
			{				
				return m_fahrstreifen2;
			}
			case 3:
			{				
				return m_fahrstreifen3;
			}
			case 4:
			case 5:
			case 6:
			case 7:
			{				
				return m_fahrstreifen4;
			}
		}
		return null;
	}

	/**
	 * bernimmt die bergebenen Grenzwerte in die interne Struktur der Parameter fr Fahrstreifen 2.
	 * 
	 * @param vStauEin Grenzwert vStauEin.
	 * @param vStauAus Grenzwert vStauAus.
	 * @param vDiffStau Grenzwert vDiffStau.
	 * @param qKfzStau Grenzwert qKfzStau.
	 */
	public void setWerteFahrstreifen2(int vStauEin, int vStauAus, int vDiffStau, int qKfzStau)
	{
		if (m_fahrstreifen2 == null)
			m_fahrstreifen2 = new ParameterStaugeschwFahrstreifen();
		
		m_fahrstreifen2.setQKfzStau(qKfzStau);
		m_fahrstreifen2.setVDiffStau(vDiffStau);
		m_fahrstreifen2.setVStauAus(vStauAus);
		m_fahrstreifen2.setVStauEin(vStauEin);
	}


	/**
	 * bernimmt die bergebenen Grenzwerte in die interne Struktur der Parameter fr Fahrstreifen 3.
	 * 
	 * @param vStauEin Grenzwert vStauEin.
	 * @param vStauAus Grenzwert vStauAus.
	 * @param vDiffStau Grenzwert vDiffStau.
	 * @param qKfzStau Grenzwert qKfzStau.
	 */
	public void setWerteFahrstreifen3(int vStauEin, int vStauAus, int vDiffStau, int qKfzStau)
	{
		if (m_fahrstreifen3 == null)
			m_fahrstreifen3 = new ParameterStaugeschwFahrstreifen();
		
		m_fahrstreifen3.setQKfzStau(qKfzStau);
		m_fahrstreifen3.setVDiffStau(vDiffStau);
		m_fahrstreifen3.setVStauAus(vStauAus);
		m_fahrstreifen3.setVStauEin(vStauEin);
	}

	/**
	 * bernimmt die bergebenen Grenzwerte in die interne Struktur der Parameter fr Fahrstreifen 4.
	 * 
	 * @param vStauEin Grenzwert vStauEin.
	 * @param vStauAus Grenzwert vStauAus.
	 * @param vDiffStau Grenzwert vDiffStau.
	 * @param qKfzStau Grenzwert qKfzStau.
	 */
	public void setWerteFahrstreifen4(int vStauEin, int vStauAus, int vDiffStau, int qKfzStau)
	{
		if (m_fahrstreifen4 == null)
			m_fahrstreifen4 = new ParameterStaugeschwFahrstreifen();
		
		m_fahrstreifen4.setQKfzStau(qKfzStau);
		m_fahrstreifen4.setVDiffStau(vDiffStau);
		m_fahrstreifen4.setVStauAus(vStauAus);
		m_fahrstreifen4.setVStauEin(vStauEin);
	}

	/**
	 * Gibt die Parameterversorgung fr 2 Fahrstreifen zurck.
	 * 
	 * @return Parameterversorgung fr 2 Fahrstreifen.
	 */
	public ParameterStaugeschwFahrstreifen getFahrstreifen2()
	{
		return m_fahrstreifen2;
	}

	/**
	 * Setzt die Parameterversorgung fr 2 Fahrstreifen.
	 * 
	 * @param fahrstreifen2 zu setzende Parameterversorgung fr 2 Fahrstreifen.
	 */
	public void setFahrstreifen2(ParameterStaugeschwFahrstreifen fahrstreifen2)
	{
		m_fahrstreifen2 = fahrstreifen2;
	}

	/**
	 * Gibt die Parameterversorgung fr 3 Fahrstreifen zurck.
	 * 
	 * @return Parameterversorgung fr 3 Fahrstreifen.
	 */
	public ParameterStaugeschwFahrstreifen getFahrstreifen3()
	{
		return m_fahrstreifen3;
	}

	/**
	 * Setzt die Parameterversorgung fr 3 Fahrstreifen.
	 * 
	 * @param fahrstreifen3 zu setzende Parameterversorgung fr 3 Fahrstreifen.
	 */
	public void setFahrstreifen3(ParameterStaugeschwFahrstreifen fahrstreifen3)
	{
		m_fahrstreifen3 = fahrstreifen3;
	}

	/**
	 * Gibt die Parameterversorgung fr 4 Fahrstreifen zurck.
	 * 
	 * @return Parameterversorgung fr 4 Fahrstreifen.
	 */
	public ParameterStaugeschwFahrstreifen getFahrstreifen4()
	{
		return m_fahrstreifen4;
	}

	/**
	 * Setzt die Parameterversorgung fr 4 Fahrstreifen.
	 * 
	 * @param fahrstreifen4 zu setzende Parameterversorgung fr 4 Fahrstreifen.
	 */
	public void setFahrstreifen4(ParameterStaugeschwFahrstreifen fahrstreifen4)
	{
		m_fahrstreifen4 = fahrstreifen4;
	}
	
  /**
	 * Innere Klasse zum Speichern der Parameter pro Fahrstreifen.<p>
	 * ber Getter und Setter kann auf die Daten zugegriffen werden.
	 * 
   * @version $Revision: 1.3 $ / $Date: 2013/02/14 07:32:49 $ / ($Author: Asal $)
	 * @author  Swarco Traffic Systems GmbH
	 * @author  Matthias Obert
	 */
  public class ParameterStaugeschwFahrstreifen
  {
  	/**
  	 * Einschaltkriterium Prognosegeschwindigkeit in km/h.
   	 */
  	protected int  m_vStauEin;
  	/**
  	 * Ausschaltkriterium Prognosegeschwindigkeit in km/h.
  	 */
  	protected int  m_vStauAus;
  	/**
  	 * Einschaltkriterium Differenzgeschwindigkeit in km/h.
  	 */
  	protected int  m_vDiffStau;
  	/**
  	 * Einschaltkriterium Verkehrsstrke Kfz in Fz/h.
  	 */
  	protected int  m_qKfzStau;

  
		/**
		 * Default-Konstruktor ohne weitere Funktion.
		 */
  	public ParameterStaugeschwFahrstreifen()
		{
		}

  	/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() 
		{
			StringBuffer string = new StringBuffer();
			
			string.append(" vStauEin " + m_vStauEin +
					          " vStauAus " + m_vStauAus +
					          " vDiffStau " + m_vDiffStau +
					          " qKfzStau " + m_qKfzStau);
			
			return string.toString();
	  }

		/**
		 * Gibt das Einschaltkriterium Prognosegeschwindigkeit in km/h zurck.
		 * 
		 * @return Einschaltkriterium Prognosegeschwindigkeit in km/h.
		 */
		public int getVStauEin()
		{
			return m_vStauEin;
		}

		/**
		 * Setzt das Einschaltkriterium Prognosegeschwindigkeit in km/h.
		 * 
		 * @param stauEin zu setzende Einschaltkriterium Prognosegeschwindigkeit in km/h.
		 */
		public void setVStauEin(int stauEin)
		{
			m_vStauEin = stauEin;
		}

		/**
		 * Gibt das Ausschaltkriterium Prognosegeschwindigkeit in km/h zurck.
		 * @return Ausschaltkriterium Prognosegeschwindigkeit in km/h.
		 */
		public int getVStauAus()
		{
			return m_vStauAus;
		}

		/**
		 * Setzt das Ausschaltkriterium Prognosegeschwindigkeit in km/h.
		 * @param stauAus zu setzende Ausschaltkriterium Prognosegeschwindigkeit in km/h.
		 */
		public void setVStauAus(int stauAus)
		{
			m_vStauAus = stauAus;
		}

		/**
		 * Gibt das Einschaltkriterium Differenzgeschwindigkeit in km/h zurck.
		 * 
		 * @return Einschaltkriterium Differenzgeschwindigkeit in km/h.
		 */
		public int getVDiffStau()
		{
			return m_vDiffStau;
		}

		/**
		 * Setzt das Einschaltkriterium Differenzgeschwindigkeit in km/h.
		 * 
		 * @param diffStau zu setzendes Einschaltkriterium Differenzgeschwindigkeit in km/h.
		 */
		public void setVDiffStau(int diffStau)
		{
			m_vDiffStau = diffStau;
		}

		/**
		 * Gibt das Einschaltkriterium Verkehrsstrke Kfz in Fz/h zurck.
		 * 
		 * @return Einschaltkriterium Verkehrsstrke Kfz in Fz/h.
		 */
		public int getQKfzStau()
		{
			return m_qKfzStau;
		}

		/**
		 * Setzt das Einschaltkriterium Verkehrsstrke Kfz in Fz/h.
		 * 
		 * @param kfzStau zu setzendes Einschaltkriterium Verkehrsstrke Kfz in Fz/h.
		 */
		public void setQKfzStau(int kfzStau)
		{
			m_qKfzStau = kfzStau;
		}
  }

	/**
	 * Gibt den Grenzwert fr den Gteindex zurck.
	 * @return Grenzwert fr den Gteindex.
	 */
	public float getGuete()
	{
		return m_guete;
	}
}