/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.stau;

import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.stealgsba.algorithmen.lib.AnzeigeEigenschaftIst.Status;
import de.bsvrz.ste.stealgsba.algorithmen.lib.AnzeigeQuerschnittEigenschaftIst.BetriebsZustand;
import de.bsvrz.ste.stealgsba.algorithmen.lib.DefinitionenGlobal;
import de.bsvrz.ste.stealgsba.algorithmen.lib.EinzelsituationVorstufeEmpfang;
import de.bsvrz.ste.stealgsba.algorithmen.lib.INeueDatenListener;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SenderOhneStatusRueckmeldung;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SimpleSenderMitWartenAufParametrierung;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SituationAnalyseMq;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SituationAnalyseMqEmpfang;
import de.bsvrz.ste.stealgsba.algorithmen.lib.StaukriteriumErgebnis;
import de.bsvrz.ste.stealgsba.algorithmen.lib.StaukriteriumErgebnisEmpfang;
import de.bsvrz.ste.stealgsba.algorithmen.lib.TestTools;
import de.bsvrz.ste.stealgsba.algorithmen.stau.ParameterStauFsOrientiert.GrenzwerteGeschwindigkeit;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.AnzeigenDatensatz;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.EinzelMassnahmeAlternative;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.Massnahmenanforderung;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.exceptions.SenderException;
import de.bsvrz.sys.funclib.vewdynobj.VerwaltungDynObj;

/**
 * Klasse fr die Durchfhrung der Prfflle 13.8 und 13.9 wie in der Prfspezifikation beschrieben.
 *
 * @version $Revision: 1.2 $ / $Date: 2015/09/30 11:37:56 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class StauTest2 implements INeueDatenListener
{
  /**
   * Datenverteilerverbindung.
   */
  private static ClientDavInterface m_connection = null;
  /**
   * Aktuelles Datenmodell.
   */
  private static DataModel m_dataModel;
  
  /**
   * Typ der Ursacheneinheit.
   */
  private static String m_typUe;
  /**
   * Pid der Ursacheneinheit.
   */
  private static String m_pidUe;
  /**
   * Name der Ursacheneinheit.
   */
  private static String m_nameUe;
  /**
   * Pid der Streckenbeeinflussungsanlage.
   */
  private static String m_anlage;
  /**
   * Synchronisierungsobjekt.
   */
  private static Object m_objekt;
  /**
   * Pid der Anzeige.
   */
  private static String m_pidAnzeige;
  /**
   * Liste mit AnzeigeInhalten von Anzeigeinhalten.
   */
  private static List<SystemObject> m_anzeigeInhalte = new ArrayList<SystemObject>();
  /**
   * Referenz auf AnzeigeQuerschnitt.
   */
  private static SystemObject m_anzeigeQuerschnitt;
  
  private static SystemObject m_wvzInhaltDunkel;
  private static SystemObject m_wvzInhalt120;
  private static SystemObject m_wvzInhalt100;
  private static SystemObject m_wvzInhalt80;
  private static SystemObject m_wvzInhalt60;
  private static SystemObject m_wvzInhalt50;
  private static SystemObject m_wvzInhalt40;
  private static SystemObject m_wvzInhaltKreuz;
  private static SystemObject m_wvzInhaltPfeilL;
  private static SystemObject m_wvzInhaltPfeilR;

  /**
   * Pid des Messquerschnitt.
   */
  private static String m_pidMessQuerschnitt;	
  /**
   * Datensatz Einzelsituationvorstufe.
   */
  private static ResultData m_DatensatzEinzelsituationVorstufe = null;
	/**
	 * Analysedaten.
	 */
	private static SituationAnalyseMq m_analyseMq = null;	
  /**
   * Zugriff auf die Instanz des Algorithmus.
   */
  private static StauAlg m_alg;
  /**
   * Zugriff auf die Instanz der Verwaltung dynamischer Objekte.
   */
  private static VerwaltungDynObj m_vdo;  
	/**
	 * Liste mit Messquerschnitten.
	 */
	private static List<SystemObject> m_messquerschnitte = new ArrayList<SystemObject>();	
	/**
	 * Liste mit Anzeigen.
	 */
	private static List<SystemObject> m_anzeigen = new ArrayList<SystemObject>();	
  /**
   * Flag Daten erhalten.
   */
  private static boolean m_datenErhalten = false;
	/**
	 * Flag Analysedaten erhalten.
	 */
	private static boolean m_analyseMqErhalten = false;
	/**
	 * Ergebnisse der Staukriterien.
	 */
	private static StaukriteriumErgebnis m_staukriterium = null;
	/**
	 * Definition der Pruefflle.
	 *
	 * @version $Revision: 1.2 $ / $Date: 2015/09/30 11:37:56 $ / ($Author: Asal $)
	 *
	 * @author Swarco Traffic Systems GmbH
	 * @author Rolf Asal
	 *
	 */
	public enum Prueffall
	{
		/**
		 * Prffall 1.
		 */
		PRF1,
		/**
		 * Prffall 2 Parametrierung.
		 */
		PRF2;
	}
	
  /**
   * Initialisiert die Testumgebung.
   * @throws Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
  	// Globale Daten setzen
    m_pidAnzeige = "anzeige.A8.6101.HFB.W.anzeige1";
    m_pidMessQuerschnitt = "mq.MQ.A8.6041.HFB.O"; 	// M10 FR Mnchen
    m_anlage = DefinitionenGlobal.getAnlage();
    m_typUe = "typ.sswSbaUeStau";
    m_pidUe = "bw.uz.A8.UeStau.JUnitTest2";
    m_nameUe = "UrsachenEinheitStau.JUnitTest2";

    // Debug-Level setzen    
    Debug.setLoggerLevel("DEFAULT", Debug.OFF);
    Debug.setHandlerLevel("StdErr", Debug.OFF);
    
    // Datenverteilerverbindung aufbauen    
    ClientDavParameters parameters = new ClientDavParameters();
    parameters.setDavCommunicationAddress(DefinitionenGlobal.getDavCommunicationAddress());
    parameters.setDavCommunicationSubAddress(DefinitionenGlobal.getDavCommunicationSubAddress());
    parameters.setUserName(DefinitionenGlobal.getUserName());
    parameters.setUserPassword(DefinitionenGlobal.getUserPassword());

    m_connection = new ClientDavConnection(parameters);
    m_connection.connect();
    m_connection.login();
    m_dataModel = m_connection.getDataModel();
    
    // Synchronisierungsobjekte erstellen
    m_objekt = new Object();
    
    // Aufrufparameter definieren
    String[] argumente = 
    {
     	DefinitionenGlobal.getArgumentDav(),
     	DefinitionenGlobal.getArgumentBenutzer(),
     	DefinitionenGlobal.getArgumentDebugLevelStdErrText(),
     	DefinitionenGlobal.getArgumentAuthentifizierung(),
  		"-Anlage=" + DefinitionenGlobal.getAnlage(),
  		"-AspektVersand=asp.sswSbaStau",
  		DefinitionenGlobal.getMaxZyklusDauer(),
  		DefinitionenGlobal.getZyklusLaenge(),
  		DefinitionenGlobal.getDeltaPruefen(),
  		"-Spontan=true"
    };
    
    // Objekte erstellen
		m_messquerschnitte.add(m_dataModel.getObject(m_pidMessQuerschnitt));

		m_anzeigen.add(m_dataModel.getObject("anzeige.A8.6041.HFB.O.anzeige1"));
		m_anzeigen.add(m_dataModel.getObject("anzeige.A8.6041.HFB.O.anzeige2"));
		m_anzeigen.add(m_dataModel.getObject("anzeige.A8.6041.HFB.O.anzeige3"));
		m_anzeigen.add(m_dataModel.getObject("anzeige.A8.6041.HFB.O.anzeige4"));
				
		m_anzeigeQuerschnitt = m_dataModel.getObject("aq.A8.6041.HFB.O");
		
		m_wvzInhaltKreuz  = m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsFahrstreifenGesperrt");
		m_wvzInhaltPfeilL = m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.nichtTlsFahrstreifenWechselnNachLinks");
		m_wvzInhaltPfeilR = m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.nichtTlsFahrstreifenWechselnNachRechts");
		m_wvzInhaltDunkel = m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.Dunkel");
		m_wvzInhalt120 = m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst120");
		m_wvzInhalt100 = m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst100");
		m_wvzInhalt80 = m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst80");
		m_wvzInhalt60 = m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst60");
		m_wvzInhalt50 = m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst50");
		m_wvzInhalt40 = m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst40");

		// Algorithmus instanziieren und starten
    m_alg = new StauAlg();
    
    StandardApplicationRunner.run(m_alg, argumente);
    
    // UE erzeugen und in Menge legen            
    m_vdo = new VerwaltungDynObj();
    m_vdo.setConnection(m_connection);
    
    try
		{
    	m_vdo.reduziereMenge(m_pidUe, "UrsachenEinheiten", m_anlage);
			m_vdo.loescheObjekt(m_pidUe);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

    try
		{
			m_vdo.erzeugeObjekt(m_typUe, m_pidUe, m_nameUe, null);
			m_vdo.erweitereMenge(m_pidUe, "UrsachenEinheiten", m_anlage);
		}
		catch (Exception e1)
		{
			e1.printStackTrace();
			
      fail("Ursacheneinheit konnte nicht erstellt werden");
		}
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  	m_vdo.reduziereMenge(m_pidUe, "UrsachenEinheiten", m_anlage);
  	m_vdo.loescheObjekt(m_pidUe);

  	m_connection.disconnect(false, "");    
  }


  @Before
  public void setUp() throws Exception
  {
  }

  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * In diesem Testfall werden die in der Prfspezifikation definierten Testflle
   * fr den Prffall 13.8 durchgefhrt.
   */
  @Test
  public final void prueffall_13_8 ()
  {
  	Prueffall prueffall = Prueffall.PRF1;
  	
  	prueffall_13_8_Vorbereitung();
  	prueffall_13_8_Parametrierung(prueffall);
  	prueffall_13_8_Test_1();
  	prueffall_13_8_Test_2();
  	prueffall_13_8_Test_3();
  	prueffall_13_8_Test_4();
  	prueffall_13_8_Test_5();
  	prueffall_13_8_Test_6();
  	prueffall_13_8_Test_7();
  	prueffall_13_8_Test_8();
  	prueffall_13_8_Test_9();
  	prueffall_13_8_Test_10();
  	prueffall_13_8_Test_11();
  	prueffall_13_8_Test_12();
  	prueffall_13_8_Test_13();
  	prueffall_13_8_Test_14();
  	prueffall_13_8_Test_15();
  	prueffall_13_8_Test_16();
  	prueffall_13_8_Test_17();
  	prueffall_13_8_Test_18();
  	prueffall_13_8_Test_19();
  	prueffall_13_8_Test_20();
  	prueffall_13_8_Test_21();
  	prueffall_13_8_Test_22();
  	prueffall_13_8_Test_23();
  	prueffall_13_8_Test_24();
  	prueffall_13_8_Test_25();
  	prueffall_13_8_Test_26();
  	prueffall_13_8_Test_27();
  	prueffall_13_8_Test_28();
  	prueffall_13_8_Test_29();
  	prueffall_13_8_Test_30();
  	prueffall_13_8_Test_31();
  	prueffall_13_8_Test_32();
  }

//  /**
//   * In diesem Testfall werden die in der Prfspezifikation definierten Testflle
//   * fr den Prffall 13.9 durchgefhrt.
//   */
//  @Test
//  public final void prueffall_13_9 ()
//  {
//  	Prueffall prueffall = Prueffall.PRF2;
//  	
//  	prueffall_13_8_Vorbereitung();
//  	prueffall_13_8_Parametrierung(prueffall);
//  	prueffall_13_8_Test_1();
//  	prueffall_13_8_Test_2();
//  	prueffall_13_8_Test_3();
//  	prueffall_13_8_Test_4();
//  	prueffall_13_8_Test_5();
//  }

  /**
   * In diesem Testfall wird berprft, ob die fr den JUnitTest notwendige Ursacheneinheit
   * korrekt erstellt wurde und vom Steuerungsalgorithmus bearbeitet wird. berprft wird
   * hierzu der Empfang der Statusinformation "EinzelsituationVorstufe", die nach dem Berechnen
   * versendet wird.
   */
  private void prueffall_13_8_Vorbereitung ()
  {
  	DynamicObject ue = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUe);
  	
    EinzelsituationVorstufeEmpfang esve = new EinzelsituationVorstufeEmpfang(m_connection, ue);
    esve.addListener(this);
    
    StaukriteriumErgebnisEmpfang skee = new StaukriteriumErgebnisEmpfang(m_connection, ue);
    skee.addListener(this);
    
    m_alg.starteBerechnung();
    
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(65000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
    		
		if (!m_datenErhalten)
			fail ("Vorbereitung EinzelsituationVorstufe nicht empfangen, Ursacheneinheit nicht korrekt erstellt");

    SituationAnalyseMqEmpfang samqe = new SituationAnalyseMqEmpfang(m_connection, ue);
    samqe.addListener(this);

  }
  
  /**
   * In diesem Tesfall werden folgende Attribitgruppen der Ursacheneinheit parametriert.<p>
   * - atg.sswSbaEinzelMassnahme<br>
   * - atg.sswSbaUrsachenEinheit<br>
   * - atg.sswSbaUeMqZuordnung<p>
   */
  private void prueffall_13_8_Parametrierung(Prueffall prueffall)
  {
  	DynamicObject ue = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUe);

  	SystemObject anzeige = m_dataModel.getObject(m_pidAnzeige);
  	
  	Data atgParam = erzeugeAtgParameterStauFsOrientiert(prueffall);
  	Data atgEm = erzeugeAtgEinzelMassnahme(erzeugeMassnahmenAnforderung(anzeige));
  	Data atgUe = erzeugeAtgUrsachenEinheit();
  	Data atgMq = erzeugeAtgMqZuordnung();
  	Data atgSk = erzeugeAtgParameterStaukriterium(prueffall);
  	
		try
		{
			parametriereDynObjUrsachenEinheit(ue, atgParam, atgEm, atgUe, atgMq, atgSk);
		}
		catch (Exception e)
		{
			fail ("Ursacheneinheit konnte nicht parametriert werden");
		}

		m_connection.sleep(25000);
  }
  
  /**
   * In dieser Methode wird der Testfall 13.8 (1) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_1()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;
		
		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 48;
		vgKfz[2] = 56;
		vgKfz[3] = 64;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();
			
			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhaltDunkel);
			m_anzeigeInhalte.add(m_wvzInhaltDunkel);
			m_anzeigeInhalte.add(m_wvzInhaltDunkel);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
				
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(20000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_staukriterium != null)
		{
			if (m_staukriterium.getStaukriterium6().getZStau() == 1)
			{
				fail("Test 1 -> Einschaltkriterium nicht korrekt");					
			}
			
			if (m_staukriterium.getStaukriterium6().getZNichtStau() == 0)
			{
				fail("Test 1 -> Ausschaltkriterium nicht korrekt");					
			}
		}
				
		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 1 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  

  /**
   * In dieser Methode wird der Testfall 13.8 (2) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_2()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 48;
		vgKfz[2] = 56;
		vgKfz[3] = 63;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

//		if (m_staukriteriumErhalten)
//		{
//			if (m_staukriterium != null)
//			{
//				if (m_staukriterium.getStaukriterium6().getZStau() == 1)
//				{
//					fail("Test 2 -> Einschaltkriterium nicht korrekt");					
//				}
//				
//				if (m_staukriterium.getStaukriterium6().getZNichtStau() == 1)
//				{
//					fail("Test 2 -> Ausschaltkriterium nicht korrekt");					
//				}
//			}
//		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 2 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }

  /**
   * In dieser Methode wird der Testfall 13.8 (3) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_3()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 48;
		vgKfz[2] = 55;
		vgKfz[3] = 64;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

//		if (m_staukriteriumErhalten)
//		{
//			if (m_staukriterium != null)
//			{
//				if (m_staukriterium.getStaukriterium6().getZStau() == 0)
//				{
//					fail("Test 3 -> Einschaltkriterium nicht korrekt");					
//				}
//				
//				if (m_staukriterium.getStaukriterium6().getZNichtStau() == 1)
//				{
//					fail("Test 3 -> Ausschaltkriterium nicht korrekt");					
//				}
//			}
//		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 3 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
	
  /**
   * In dieser Methode wird der Testfall 13.8 (4) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_4()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 47;
		vgKfz[2] = 56;
		vgKfz[3] = 64;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

//		if (m_staukriteriumErhalten)
//		{
//			if (m_staukriterium != null)
//			{
//				if (m_staukriterium.getStaukriterium6().getZStau() == 0)
//				{
//					fail("Test 4 -> Einschaltkriterium nicht korrekt");					
//				}
//				
//				if (m_staukriterium.getStaukriterium6().getZNichtStau() == 1)
//				{
//					fail("Test 4 -> Ausschaltkriterium nicht korrekt");					
//				}
//			}
//		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 4 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }

  /**
   * In dieser Methode wird der Testfall 13.8 (5) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_5()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 47;
		vgKfz[2] = 56;
		vgKfz[3] = 63;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

//		if (m_staukriteriumErhalten)
//		{
//			if (m_staukriterium != null)
//			{
//				if (m_staukriterium.getStaukriterium6().getZStau() == 1)
//				{
//					fail("Test 5 -> Einschaltkriterium nicht korrekt");					
//				}
//				
//				if (m_staukriterium.getStaukriterium6().getZNichtStau() == 0)
//				{
//					fail("Test 5 -> Ausschaltkriterium nicht korrekt");					
//				}
//			}
//		}
    
		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 5 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }

  /**
   * In dieser Methode wird der Testfall 13.8 (6) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_6()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;
		
		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 47;
		vgKfz[2] = 55;
		vgKfz[3] = 63;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_staukriterium != null)
		{
			if (m_staukriterium.getStaukriterium6().getZStau() == 0)
			{
				fail("Test 6 -> Einschaltkriterium nicht korrekt");					
			}
			
			if (m_staukriterium.getStaukriterium6().getZNichtStau() == 1)
			{
				fail("Test 6 -> Ausschaltkriterium nicht korrekt");					
			}
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 6 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  
  /**
   * In dieser Methode wird der Testfall 13.8 (7) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_7()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 50;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1301;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 7 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }

  /**
   * In dieser Methode wird der Testfall 13.8 (8) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_8()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 50;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1799;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 8 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (9) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_9()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 51;
		vgKfz[2] = 50;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1801;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 9 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (10) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_10()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 51;
		vgKfz[2] = 51;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1801;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 10 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  
  /**
   * In dieser Methode wird der Testfall 13.8 (11) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_11()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;
		
		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 51;
		vgKfz[2] = 51;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1801;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();
			
			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt60);
			m_anzeigeInhalte.add(m_wvzInhalt60);
			m_anzeigeInhalte.add(m_wvzInhalt60);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
				
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(20000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
				
		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 11 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  
  /**
   * In dieser Methode wird der Testfall 13.8 (12) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_12()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 47;
		vgKfz[2] = 55;
		vgKfz[3] = 64;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1301;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();
			
			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhaltDunkel);
			m_anzeigeInhalte.add(m_wvzInhaltDunkel);
			m_anzeigeInhalte.add(m_wvzInhaltDunkel);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 12 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (13) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_13()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 50;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 13 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (14) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_14()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 48;
		vgKfz[2] = 55;
		vgKfz[3] = 63;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1801;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 14 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (15) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_15()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 51;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1799;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 15 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (16) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_16()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 51;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1799;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();
			
			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt60);
			m_anzeigeInhalte.add(m_wvzInhalt60);
			m_anzeigeInhalte.add(m_wvzInhalt60);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 16 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (17) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_17()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 48;
		vgKfz[2] = 56;
		vgKfz[3] = 56;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();
			
			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt120);
			m_anzeigeInhalte.add(m_wvzInhalt120);
			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 17 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (18) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_18()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 48;
		vgKfz[2] = 56;
		vgKfz[3] = 55;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1301;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 18 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (19) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_19()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 48;
		vgKfz[2] = 55;
		vgKfz[3] = 55;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1301;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 19 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (20) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_20()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 51;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();

			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt40);
			m_anzeigeInhalte.add(m_wvzInhalt40);
			m_anzeigeInhalte.add(m_wvzInhalt40);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 20 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (21) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_21()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 40;
		vgKfz[2] = 45;
		vgKfz[3] = 50;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();

			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt100);
			m_anzeigeInhalte.add(m_wvzInhalt100);
			m_anzeigeInhalte.add(m_wvzInhalt100);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 21 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (22) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_22()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 39;
		vgKfz[2] = 44;
		vgKfz[3] = 49;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 22 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (23) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_23()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 50;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();

			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt60);
			m_anzeigeInhalte.add(m_wvzInhalt60);
			m_anzeigeInhalte.add(m_wvzInhalt60);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 23 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (24) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_24()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 41;
		vgKfz[2] = 41;
		vgKfz[3] = 41;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();

			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt80);
			m_anzeigeInhalte.add(m_wvzInhalt80);
			m_anzeigeInhalte.add(m_wvzInhalt80);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 24 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (25) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_25()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 39;
		vgKfz[2] = 41;
		vgKfz[3] = 39;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1301;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 25 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (26) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_26()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 51;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1799;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();

			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhaltDunkel);
			m_anzeigeInhalte.add(m_wvzInhaltDunkel);
			m_anzeigeInhalte.add(m_wvzInhaltDunkel);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 26 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (27) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_27()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 51;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1799;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();

			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt40);
			m_anzeigeInhalte.add(m_wvzInhalt40);
			m_anzeigeInhalte.add(m_wvzInhalt40);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 27 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (28) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_28()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 31;
		vgKfz[2] = 31;
		vgKfz[3] = 31;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();

			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt60);
			m_anzeigeInhalte.add(m_wvzInhalt60);
			m_anzeigeInhalte.add(m_wvzInhalt60);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 28 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (29) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_29()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 29;
		vgKfz[2] = 29;
		vgKfz[3] = 29;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 29 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (30) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_30()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 51;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1801;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 30 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  /**
   * In dieser Methode wird der Testfall 13.8 (31) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_31()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 50;
		vgKfz[2] = 50;
		vgKfz[3] = 51;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1299;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();

			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt40);
			m_anzeigeInhalte.add(m_wvzInhalt40);
			m_anzeigeInhalte.add(m_wvzInhalt40);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 31 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }

  /**
   * In dieser Methode wird der Testfall 13.8 (32) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_8_Test_32()
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int vgKfz[] = new int[5];
		
		vgKfz[0] = 0;
		vgKfz[1] = 29;
		vgKfz[2] = 29;
		vgKfz[3] = 31;
		vgKfz[4] = 100;
		
		int vKfz = 100;
		int qB   = 1301;
		
		try
		{
			Data verkehrsdatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, vKfz, qB);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SystemObject mq = m_messquerschnitte.get(i);
				
				SenderOhneStatusRueckmeldung.source(m_connection, mq, dd, verkehrsdatenMq);				

				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"), m_dataModel.getAspect("asp.analyse"));
				
				List<SystemObject> _fahrstreifen = bestimmeFahrstreifenMq(mq);
				if (_fahrstreifen != null)
				{
					Data verkehrsdatenFs = null;
					
					for (int j = 0; j < _fahrstreifen.size(); j++)
					{
						verkehrsdatenFs = erzeugeAtgVerkehrsDatenKurzZeitFs(vgKfz[j]);
						
						SenderOhneStatusRueckmeldung.source(m_connection, _fahrstreifen.get(j), dd, verkehrsdatenFs);
					}
				}
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		/**
		 * Anzeigeinhalte vorgeben. von rechts nach links
		 */
		try
		{
			m_anzeigeInhalte.clear();

			m_anzeigeInhalte.add(m_wvzInhaltKreuz);
			m_anzeigeInhalte.add(m_wvzInhalt40);
			m_anzeigeInhalte.add(m_wvzInhalt40);
			m_anzeigeInhalte.add(m_wvzInhalt40);
//			m_anzeigeInhalte.add(m_wvzInhalt120);
//			m_anzeigeInhalte.add(m_wvzInhalt120);

			Data aqIst = erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand.AQNORMALBETRIEB, Status.OK);

			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
			
			SenderOhneStatusRueckmeldung.source(m_connection, m_anzeigeQuerschnitt, dd, aqIst);											
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				SystemObject anzeige = m_anzeigen.get(i);
				
				Data anzIst = erzeugeAtgAnzeigeEigenschaftIst(Status.OK, m_anzeigeInhalte.get(i));
				
				dd = new DataDescription(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
				
				SenderOhneStatusRueckmeldung.source(m_connection, anzeige, dd, anzIst);											
			}
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (!status)
			{
				fail ("Test 32 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  
	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.INeueDatenListener#neueDaten(de.bsvrz.dav.daf.main.ResultData)
	 */
	@Override
	public void neueDaten(ResultData rdata)
	{
		System.out.println("StauTest2.neueDaten() EinzelsituationVorstufe " + rdata.getObject().getPid());
		
		m_datenErhalten = true;
		m_DatensatzEinzelsituationVorstufe = rdata;
		
		synchronized (m_objekt)
		{
			m_objekt.notify();					
		}
	}
  
  /**
   * Erzeugt eine Liste mit Fahrstreifen fr die bergebene Liste der Messquerschnitte.
   * 
   * @param mqs Liste mit Messquerschnitten.
   * @return Liste mit Fahrstreifenobjekte.
   */
  private static List<SystemObject> bildeListeFahrstreifen(List<SystemObject> mqs)
	{
		if (mqs != null)
		{
			List<SystemObject> listeFs = new ArrayList<SystemObject>();
			
			for (int i = 0; i < mqs.size(); i++)
			{
				listeFs.addAll(bestimmeFahrstreifenMq(mqs.get(i)));
			}

			return listeFs;
		}
		return null;
	}
  
  /**
   * Bestimmt die Fahrstreifen des bergebenen Messquerschnitts.
   * 
   * @param mq Messquerschnitt.
   * @return Liste der Fahrstreifenobjekte.
   */
  private static List<SystemObject> bestimmeFahrstreifenMq(SystemObject mq)
	{
  	List<SystemObject> fahrstreifen = new ArrayList<SystemObject>();
  	
		if (mq != null)
		{			
			HashMap<Integer, SystemObject> fsMap = new HashMap<Integer, SystemObject>();
			List<Integer> _sortedList = new ArrayList<Integer>();

			ConfigurationObject coObj = (ConfigurationObject)mq;
				
			List<SystemObject> listeFs = coObj.getNonMutableSet("FahrStreifen").getElements();
			for (int i = 0; i < listeFs.size(); i++)
			{
				ConfigurationObject coFs = (ConfigurationObject)listeFs.get(i);
				
				Data data = coFs.getConfigurationData(m_dataModel.getAttributeGroup("atg.fahrStreifen"));

				int lage = data.getUnscaledValue("Lage").intValue();
				fsMap.put(lage, listeFs.get(i));
			}
			
			// Liste aufsteigend sortieren
			_sortedList.addAll(fsMap.keySet());
			Collections.sort(_sortedList);		

//				System.out.println("-----------> " + _sortedList);

			// Sortierte Ergebnisliste EreignisTyp bilden
			for (int i = 0; i < _sortedList.size(); i++)
			{
				fahrstreifen.add(fsMap.get(_sortedList.get(i)));
			} 

			return fahrstreifen;
		}

		return null;
	}

	/**
	 * Bestimmt den aktuellen Zustand der Einzelsituationvorstufe des empfangenen Datensatzes.
	 * 
	 * @return true wenn Einzelsituationvorstufe wahr, ansonsten false.
	 */
	private boolean bestimmeStatusEinzelsituationVorstufe()
	{
		if (m_DatensatzEinzelsituationVorstufe != null)
		{
			if (m_DatensatzEinzelsituationVorstufe.hasData())
			{				
				Data data = m_DatensatzEinzelsituationVorstufe.getData();
				
				return data.getScaledValue("Status").getValueText().equals("Ja");
			}
		}
		
		return false;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaEinzelMassnahme</code>
	 * und gibt ihn zurck.
	 * 
	 * @param massnahmenanforderung Daten der Massnahmeanforderung.
	 * @return Datensatz <code>atg.sswSbaEinzelMassnahme</code>.
	 */
	private Data erzeugeAtgEinzelMassnahme(Massnahmenanforderung massnahmenanforderung)
	{
		if (massnahmenanforderung != null)
		{				
			Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaEinzelMassnahme"));

			TestTools.fuelleAtgEinzelMassnahme(massnahmenanforderung, dataNeu);
			
			return dataNeu;
		}
		
		return null;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUrsachenEinheit</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaUrsachenEinheit</code>.
	 */
	private Data erzeugeAtgUrsachenEinheit()
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaUrsachenEinheit"));
		
		TestTools.fuelleAtgUrsachenEinheit(dataNeu);
		
		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeMqZuordnung</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaUeMqZuordnungt</code>.
	 */
  private Data erzeugeAtgMqZuordnung()
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaUeMqZuordnung"));

		List<SystemObject> mqs = new ArrayList<SystemObject>();
		mqs.add(m_dataModel.getObject(m_pidMessQuerschnitt));
		
		TestTools.fuelleAtgMqZuordnung(mqs, dataNeu);
		
		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.verkehrsDatenKurzZeitFs</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.verkehrsDatenKurzZeitFs</code>.
	 */
  private Data erzeugeAtgVerkehrsDatenKurzZeitFs(int vgkfz)
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs"));
		
		TestTools.fuelleAtgVerkehrsDatenKurzZeitFs(vgkfz, dataNeu);
		
		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.verkehrsDatenKurzZeitTrendExtraPolationMq</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.verkehrsDatenKurzZeitTrendExtraPolationMq</code>.
	 */
  private Data erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(int qkfz, int vkfz, int qb)
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"));
		
		TestTools.fuelleAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(qkfz, vkfz, -1, qb, -1, -1, dataNeu);
		
		return dataNeu;
	}
	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeStauBeleg</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaUeStauBeleg</code>.
	 */
  private Data erzeugeAtgParameterStaukriterium(Prueffall prueffall)
	{
		DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.sswSbaUeStaukriterium"), m_dataModel.getAspect("asp.parameterSoll"));
  	DynamicObject ue = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUe);

		// Aktuelle Parameter holen
		ResultData rdata = m_connection.getData(ue, dd, 1000);
		
		// Wert setzen
		ParameterStaukriterium parameter = new ParameterStaukriterium(rdata, true);
		
		if (prueffall.equals(Prueffall.PRF1))
		{
			parameter.getStaukriterium1().getNichtStau().setVertrauenswrdigkeit(0);
			parameter.getStaukriterium1().getStau().setVertrauenswrdigkeit(0);
			
			parameter.getStaukriterium2().getNichtStau().setVertrauenswrdigkeit(0);
			parameter.getStaukriterium2().getStau().setVertrauenswrdigkeit(0);
			
			parameter.getStaukriterium3().getNichtStau().setVertrauenswrdigkeit(0);
			parameter.getStaukriterium3().getStau().setVertrauenswrdigkeit(0);
			
			parameter.getStaukriterium4().getNichtStau().setVertrauenswrdigkeit(0);
			parameter.getStaukriterium4().getStau().setVertrauenswrdigkeit(0);
			
			parameter.getStaukriterium5().getNichtStau().setVertrauenswrdigkeit(0);
			parameter.getStaukriterium5().getStau().setVertrauenswrdigkeit(0);
			
			parameter.getStaukriterium6().getNichtStau().setVertrauenswrdigkeit(1);
			parameter.getStaukriterium6().getNichtStau().setZugehrigkeit(1);			
			parameter.getStaukriterium6().getStau().setVertrauenswrdigkeit(1);
			parameter.getStaukriterium6().getStau().setZugehrigkeit(1);
		}
		
		return parameter.erzeugeDatensatz((ClientDavConnection)m_connection);
	}
  
	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeStauBeleg</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaUeStauBeleg</code>.
	 */
  private Data erzeugeAtgParameterStauFsOrientiert(Prueffall prueffall)
	{
		DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.sswSbaUeStauFsOrientiert"), m_dataModel.getAspect("asp.parameterSoll"));
  	DynamicObject ue = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUe);

		// Aktuelle Parameter holen
		ResultData rdata = m_connection.getData(ue, dd, 1000);
		
		// Wert setzen
		ParameterStauFsOrientiert parameter = new ParameterStauFsOrientiert(rdata);
		
		if (prueffall.equals(Prueffall.PRF1))
		{
			parameter.getGrenzwerte1Bis2Fs(1).setQ_f(720);
			parameter.getGrenzwerte1Bis2Fs(2).setQ_f(1300);

			parameter.getGrenzwerte3Bis5Fs(3).setQ_f(1300);
			parameter.getGrenzwerte3Bis5Fs(3).setQ_g(1800);
			parameter.getGrenzwerte3Bis5Fs(3).setX1(2);
			parameter.getGrenzwerte3Bis5Fs(3).setX2(1);
			parameter.getGrenzwerte3Bis5Fs(3).setX3(2);

			parameter.getGrenzwerte3Bis5Fs(4).setQ_f(1300);
			parameter.getGrenzwerte3Bis5Fs(4).setQ_g(1800);
			parameter.getGrenzwerte3Bis5Fs(4).setX1(2);
			parameter.getGrenzwerte3Bis5Fs(4).setX2(2);
			parameter.getGrenzwerte3Bis5Fs(4).setX3(2);

			parameter.getGrenzwerte3Bis5Fs(5).setQ_f(1300);
			parameter.getGrenzwerte3Bis5Fs(5).setQ_g(1800);
			parameter.getGrenzwerte3Bis5Fs(5).setX1(3);
			parameter.getGrenzwerte3Bis5Fs(5).setX2(3);
			parameter.getGrenzwerte3Bis5Fs(5).setX3(3);

			GrenzwerteGeschwindigkeit gw = parameter.new GrenzwerteGeschwindigkeit();
			parameter.getvEin().clear();
			
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst40"));
			gw.setFs1(30);
			gw.setFs2(30);
			gw.setFs3(30);
			gw.setFs4(30);
			gw.setFs5(30);
			parameter.getvEin().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst60"));
			gw.setFs1(30);
			gw.setFs2(30);
			gw.setFs3(30);
			gw.setFs4(30);
			gw.setFs5(30);
			parameter.getvEin().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst80"));
			gw.setFs1(40);
			gw.setFs2(40);
			gw.setFs3(40);
			gw.setFs4(40);
			gw.setFs5(40);
			parameter.getvEin().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst100"));
			gw.setFs1(40);
			gw.setFs2(45);
			gw.setFs3(50);
			gw.setFs4(50);
			gw.setFs5(50);
			parameter.getvEin().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst120"));
			gw.setFs1(48);
			gw.setFs2(56);
			gw.setFs3(56);
			gw.setFs4(56);
			gw.setFs5(56);
			parameter.getvEin().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.Dunkel"));
			gw.setFs1(48);
			gw.setFs2(56);
			gw.setFs3(64);
			gw.setFs4(64);
			gw.setFs5(64);
			parameter.getvEin().add(gw);
			
			parameter.getvAus().clear();

			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst50"));
			gw.setFs1(50);
			gw.setFs2(50);
			gw.setFs3(50);
			gw.setFs4(50);
			gw.setFs5(50);
			parameter.getvAus().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst60"));
			gw.setFs1(50);
			gw.setFs2(50);
			gw.setFs3(50);
			gw.setFs4(50);
			gw.setFs5(50);
			parameter.getvAus().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.Dunkel"));
			gw.setFs1(50);
			gw.setFs2(50);
			gw.setFs3(50);
			gw.setFs4(50);
			gw.setFs5(50);
			parameter.getvAus().add(gw);
			
			parameter.getAnzeige().clear();

			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				parameter.getAnzeige().add(m_anzeigen.get(i));				
			}
			
			parameter.setGuete(1);
		}
		else
		{
			parameter.getGrenzwerte1Bis2Fs(1).setQ_f(720);
			parameter.getGrenzwerte1Bis2Fs(2).setQ_f(1300);

			parameter.getGrenzwerte3Bis5Fs(3).setQ_f(1300);
			parameter.getGrenzwerte3Bis5Fs(3).setQ_g(1800);
			parameter.getGrenzwerte3Bis5Fs(3).setX1(2);
			parameter.getGrenzwerte3Bis5Fs(3).setX2(1);
			parameter.getGrenzwerte3Bis5Fs(3).setX3(2);

			parameter.getGrenzwerte3Bis5Fs(4).setQ_f(1300);
			parameter.getGrenzwerte3Bis5Fs(4).setQ_g(1800);
			parameter.getGrenzwerte3Bis5Fs(4).setX1(2);
			parameter.getGrenzwerte3Bis5Fs(4).setX2(2);
			parameter.getGrenzwerte3Bis5Fs(4).setX3(2);

			parameter.getGrenzwerte3Bis5Fs(5).setQ_f(1300);
			parameter.getGrenzwerte3Bis5Fs(5).setQ_g(1800);
			parameter.getGrenzwerte3Bis5Fs(5).setX1(3);
			parameter.getGrenzwerte3Bis5Fs(5).setX2(3);
			parameter.getGrenzwerte3Bis5Fs(5).setX3(3);

			GrenzwerteGeschwindigkeit gw = parameter.new GrenzwerteGeschwindigkeit();

			parameter.getvEin().clear();

			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst50"));
			gw.setFs1(30);
			gw.setFs2(30);
			gw.setFs3(30);
			gw.setFs4(30);
			gw.setFs5(30);
			parameter.getvEin().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst60"));
			gw.setFs1(35);
			gw.setFs2(35);
			gw.setFs3(35);
			gw.setFs4(35);
			gw.setFs5(35);
			parameter.getvEin().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst80"));
			gw.setFs1(45);
			gw.setFs2(45);
			gw.setFs3(45);
			gw.setFs4(45);
			gw.setFs5(45);
			parameter.getvEin().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst100"));
			gw.setFs1(40);
			gw.setFs2(45);
			gw.setFs3(50);
			gw.setFs4(50);
			gw.setFs5(50);
			parameter.getvEin().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst120"));
			gw.setFs1(48);
			gw.setFs2(56);
			gw.setFs3(56);
			gw.setFs4(56);
			gw.setFs5(56);
			parameter.getvEin().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.Dunkel"));
			gw.setFs1(48);
			gw.setFs2(56);
			gw.setFs3(64);
			gw.setFs4(64);
			gw.setFs5(64);
			parameter.getvEin().add(gw);

			parameter.getvAus().clear();

			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst50"));
			gw.setFs1(50);
			gw.setFs2(50);
			gw.setFs3(50);
			gw.setFs4(50);
			gw.setFs5(50);
			parameter.getvAus().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.tlsZulaessigeHoechst60"));
			gw.setFs1(50);
			gw.setFs2(50);
			gw.setFs3(50);
			gw.setFs4(50);
			gw.setFs5(50);
			parameter.getvAus().add(gw);
			gw = parameter.new GrenzwerteGeschwindigkeit();
			gw.setAktuelleAnzeige(m_dataModel.getObject("wvzInhalt.uz.A8.sba_stgt.Dunkel"));
			gw.setFs1(50);
			gw.setFs2(50);
			gw.setFs3(50);
			gw.setFs4(50);
			gw.setFs5(50);
			parameter.getvAus().add(gw);

			parameter.getAnzeige().clear();
			
			for (int i = 0; i < m_anzeigen.size(); i++)
			{
				parameter.getAnzeige().add(m_anzeigen.get(i));				
			}

			parameter.setGuete(1);
		}
		
		return parameter.erzeugeDatensatz((ClientDavConnection)m_connection);
	}

  /**
   * Methode erzeugt eine Manahmenanforderung mit der fr eine Anzeige ein bestimmter WVZ-Inhalt fr einen bestimmten
   * Schaltgrund angefordert wird
   * @param anzeige Anzeige die geschaltet werden soll
   * @return erzeugte Manahmenanforderung
   */
  private Massnahmenanforderung erzeugeMassnahmenAnforderung(SystemObject anzeige)
  {
    Massnahmenanforderung ma = new Massnahmenanforderung ();
    Vector<EinzelMassnahmeAlternative> einzelMassnahmeAlternative = new Vector<EinzelMassnahmeAlternative>();
    ma.setEinzelMassnahmeAlternative(einzelMassnahmeAlternative);
    
    EinzelMassnahmeAlternative alternative = new EinzelMassnahmeAlternative();
    einzelMassnahmeAlternative.add(alternative);
    
    Vector<AnzeigenDatensatz> datenSaetze = new Vector<AnzeigenDatensatz>();
    alternative.setEinzelMassnahmeAlternative(datenSaetze);
        
    AnzeigenDatensatz datensatz = new AnzeigenDatensatz();
    datenSaetze.add(datensatz);
    
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaMassnahmenAnforderung"));
		dataNeu.getArray("EinzelMassnahmeAlternative").setLength(1);
		dataNeu.getArray("EinzelMassnahmeAlternative").getItem(0).getArray("AnzeigenDatenstze").setLength(1);
		Data dataWvzInhalt = dataNeu.getArray("EinzelMassnahmeAlternative").getItem(0).getArray("AnzeigenDatenstze").getItem(0).getItem("WVZInhalt"); 

		SystemObject anzeigeInhalt = m_dataModel.getObject("bw.uz.B27.streckenBeeinflussungsAnlage.wvzInhalt.tlsGefahrenstelle");
		
		// WVZInhalt
		dataWvzInhalt.getReferenceValue("AnzeigeInhalt").setSystemObject(anzeigeInhalt);
		dataWvzInhalt.getScaledValue("Blinken").setText("Nein");
		dataWvzInhalt.getScaledValue("Blinktakt").set(-1);
		dataWvzInhalt.getArray("GrafikDarstellungen").setLength(0);
		dataWvzInhalt.getReferenceValue("Wechseltext").setText("undefiniert");
    
    datensatz.setAnzeige(anzeige);
    datensatz.setAnzeigeInhalt(anzeigeInhalt);
    datensatz.setAbsolutNotwendig(false);
    datensatz.setNotwendig(false);
    datensatz.setPrioritt(-1);
    datensatz.setVerschiebbarNachfolger(0);
    datensatz.setVerschiebbarVorgnger(0);
    datensatz.setWvzInhalt(dataWvzInhalt);
    
    return ma;
  }
  
	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.anzeigeQuerschnittEigenschaftIst</code>
	 * und gibt ihn zurck.
	 * 
	 * @param betriebszustand zu setzender Betriebszustand.
	 * @param status zu setzender Status.
	 * @return Datensatz <code>atg.anzeigeQuerschnittEigenschaftIst</code>.
	 */
  private Data erzeugeAtgAnzeigeQuerschnittEigenschaftIst(BetriebsZustand betriebszustand, Status status)
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"));

		dataNeu.getItem("Eigenschaft").getUnscaledValue("Helligkeit").set(100);
		
		if (betriebszustand.equals(BetriebsZustand.AQAUTARKBETRIEB))			
			dataNeu.getItem("Eigenschaft").getScaledValue("Betriebszustand").setText("AQAutarkbetrieb");
		else if (betriebszustand.equals(BetriebsZustand.AQNORMALBETRIEB))			
			dataNeu.getItem("Eigenschaft").getScaledValue("Betriebszustand").setText("AQNormalbetrieb");
		else if (betriebszustand.equals(BetriebsZustand.AQBLINDBETRIEB))			
			dataNeu.getItem("Eigenschaft").getScaledValue("Betriebszustand").setText("AQBlindbetrieb");
		else if (betriebszustand.equals(BetriebsZustand.AQVORORTBETRIEB))			
			dataNeu.getItem("Eigenschaft").getScaledValue("Betriebszustand").setText("AQVorortbetrieb");
		else if (betriebszustand.equals(BetriebsZustand.AQEXTERNESTEUERUNG))			
			dataNeu.getItem("Eigenschaft").getScaledValue("Betriebszustand").setText("AQExterneSteuerung");
		else
			dataNeu.getItem("Eigenschaft").getScaledValue("Betriebszustand").setText("Undefiniert");

		dataNeu.getItem("Eigenschaft").getScaledValue("Passiviert").setText("Nein");

		dataNeu.getArray("Anzeigen").setLength(0);
		
		if (status.equals(Status.GESTOERT))
			dataNeu.getScaledValue("Status").setText("Gestrt");
		else if (status.equals(Status.OK))
			dataNeu.getScaledValue("Status").setText("OK");
		else
			dataNeu.getScaledValue("Status").setText("Undefiniert");

		dataNeu.getArray("AktFehlerMeldungen").setLength(0);

		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.anzeigeEigenschaftIst</code>
	 * und gibt ihn zurck.
	 * 
	 * @param status zu setzender Status.
	 * @param anzeigeInhalt zu setzender Anzeigeinhalt.
	 * @return Datensatz <code>atg.anzeigeEigenschaftIst</code>.
	 */
  private Data erzeugeAtgAnzeigeEigenschaftIst(Status status, SystemObject anzeigeInhalt)
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst"));

		dataNeu.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setSystemObject(anzeigeInhalt);
		dataNeu.getItem("Eigenschaft").getScaledValue("Blinken").setText("Nein");
		dataNeu.getItem("Eigenschaft").getUnscaledValue("Blinktakt").set(-1);
		dataNeu.getItem("Eigenschaft").getScaledValue("Passiviert").setText("Nein");
		dataNeu.getItem("Eigenschaft").getReferenceValue("Wechseltext").setText("undefiniert");

		dataNeu.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceValue("Font").setText("undefiniert");
		dataNeu.getItem("Eigenschaft").getItem("DynamischerWechseltext").getScaledValue("TextAusrichtung").setText("Undefiniert");
		dataNeu.getItem("Eigenschaft").getItem("DynamischerWechseltext").getArray("Textdefinition").setLength(0);
		
		if (status.equals(Status.GESTOERT))
			dataNeu.getScaledValue("Status").setText("Gestrt");
		else if (status.equals(Status.OK))
			dataNeu.getScaledValue("Status").setText("OK");
		else
			dataNeu.getScaledValue("Status").setText("Undefiniert");

		dataNeu.getArray("GestrteInhalte").setLength(0);
		dataNeu.getArray("GestrteTextpositionen").setLength(0);
		dataNeu.getArray("AktFehlerMeldungen").setLength(0);

		return dataNeu;
	}
  
	/**
	 * Diese Methode parametriert das bergebene Objekt mit den bergebenen Datenstze.
	 * Wird <code>null</code> bergeben wird die entsprechende Attributgruppe nicht
	 * parametriert.
	 * 
	 * @param ue Dyn. Objekt Ursacheneinheit.
	 * @param atgSp Datensatz <code>atg.sswSbaUeStauBeleg</code>.
	 * @param atgEm Datensatz <code>atg.sswSbaEinzelMassnahme</code>.
	 * @param atgUe Datensatz <code>atg.sswSbaUrsachenEinheit</code>.
	 * @param atgmq Datensatz <code>atg.sswSbaUeMqZuordnung</code>.
	 */
	private void parametriereDynObjUrsachenEinheit(DynamicObject ue, Data atgParam, Data atgEm, Data atgUe, Data atgMq, Data atgSk)
	{
		if (ue != null)
		{
			System.out.println("Ursacheneinheit " + ue + " parametrieren");
			
			Aspect asp = m_dataModel.getAspect("asp.parameterVorgabe");

      if (atgParam != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUeStauFsOrientiert");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgParam, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }
      
      if (atgEm != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaEinzelMassnahme");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgEm, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }
      
      if (atgUe != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUrsachenEinheit");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgUe, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }

      if (atgMq != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUeMqZuordnung");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgMq, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }
      
      if (atgSk != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUeStaukriterium");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgSk, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }

		}			
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.INeueDatenListener#neueDatenSituationAnalyseMq(de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.SituationAnalyseMq)
	 */
	@Override
	public void neueDatenSituationAnalyseMq(SituationAnalyseMq analyse)
	{
		System.out.println("StauTest2.neueDatenSituationAnalyseMq()" + analyse);

		m_analyseMq = analyse;
		m_analyseMqErhalten = true;
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.INeueDatenListener#neueDatenStaukriterium(de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.StaukriteriumErgebnis)
	 */
	@Override
	public void neueDatenStaukriterium(StaukriteriumErgebnis staukriterium)
	{
		
		System.out.println("StauTest2.neueDatenStaukriterium() " + staukriterium);
		
		m_staukriterium = staukriterium;
	}
}
