/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.stau;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.stealgsba.algorithmen.lib.Anzeige;
import de.bsvrz.ste.stealgsba.algorithmen.lib.Definitionen;
import de.bsvrz.ste.stealgsba.algorithmen.lib.FahrStreifen;
import de.bsvrz.ste.stealgsba.algorithmen.lib.MessQuerschnitt;
import de.bsvrz.ste.stealgsba.algorithmen.lib.ParameterMqZuordnung;
import de.bsvrz.ste.stealgsba.algorithmen.lib.PassivierungFahrstreifen;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SituationAnalyseMq;
import de.bsvrz.ste.stealgsba.algorithmen.lib.StoerfallZustand;
import de.bsvrz.ste.stealgsba.algorithmen.lib.VerkehrsDatenKurzZeitFs;
import de.bsvrz.ste.stealgsba.algorithmen.lib.VerkehrsDatenKurzZeitMq;
import de.bsvrz.ste.stealgsba.algorithmen.lib.VerkehrsDatenKurzZeitTrendExtraPolationMq;
import de.bsvrz.ste.stealgsba.algorithmen.stau.ParameterStauGeschw.ParameterStaugeschwFahrstreifen;
import de.bsvrz.ste.stealgsba.algorithmen.stau.ParameterStauStoerfall.ParameterStauStoerfallFahrstreifen;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.UrsacheneinheitErweiterung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse implementiert die Funktionalitt der Situationsbearbeitung fr einen Algorithmus.<p>
 * Verwaltung der fr die Situationsbestimmung notwendigen Parameter und Eingangswerte.
 *
 * @version $Revision: 1.10 $ / $Date: 2015/09/28 12:01:40 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 *
 */
public class StauUe extends UrsacheneinheitErweiterung implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private static Debug m_debug = Debug.getLogger();
	/**
	 * Systemobjekt der Ursacheneinheit.
	 */
	private SystemObject m_objekt;
	/**
	 * Attributgruppe fr den Empfang der Parameter Staukriterium 1.
	 */
	private AttributeGroup m_atgParameterStauBeleg;
	/**
	 * Attributgruppe fr den Empfang der Parameter Staukriterium 2.
	 */
	private AttributeGroup m_atgParameterStauGeschw;
	/**
	 * Attributgruppe fr den Empfang der Parameter Staukriterium 3.
	 */
	private AttributeGroup m_atgParameterStauStoerfall;
	/**
	 * Attributgruppe fr den Empfang der Parameter Staukriterium.
	 */
	private AttributeGroup m_atgParameterStaukriterium;
	/**
	 * Attributgruppe fr den Versand der Analysedaten der Situationsbestimmung.
	 */
	private AttributeGroup m_atgAnalyse;
	/**
	 * Attributgruppe fr den Empfang der Parameter Zuordnung Messquerschnitt.
	 */
	private AttributeGroup m_atgParameterMqZuordnung;
	/**
	 * Attributgruppe fr den Versand der Zusammenfassung der Staukriterien.
	 */
	private AttributeGroup m_atgZusammenfassungStaukriterien;
	/**
	 * Attributgruppe fr den Versand der Ergebnisse der Staukriterien.
	 */
	private AttributeGroup m_atgErgebnisStaukriterien;
	/**
	 * Aspekt fr den Empfang der globalen Parameter.
	 */
	private Aspect m_aspParameter;
	/**
	 * Aspekt fr den Versand der Analysedaten.
	 */
	private Aspect m_aspAnalyse;
  /**
   * Parameter fr die Situationsbestimmung Staukriterium 1 Belegung.
   */
  private ParameterStauBeleg m_parameterStauBeleg;  
  /**
   * Parameter fr die Situationsbestimmung Staukriterium 2 Prognosegeschwindigkeit.
   */
  private ParameterStauGeschw m_parameterStauGeschw;  
  /**
   * Parameter fr die Situationsbestimmung Staukriterium 3 Strfall.
   */
  private ParameterStauStoerfall m_parameterStauStoerfall;
  /**
   * Parameter fr die Bewertung der Staukriterien.
   */
  private ParameterStaukriterium m_parameterStaukriterium;
  /**
   * Parameter Zuordnung Messquerschnitt.
   */
  private ParameterMqZuordnung m_parameterMqZuordnung;  
  /**
   * Zuordnung Messquerschnitt Alt.
   */
  private List<SystemObject> m_mqZuordnungAlt = new ArrayList<SystemObject>();  
  /**
   * Speicher fr die Objekte zur Verwaltung der Messquerschnittsdaten.<br>
   * Key ist die PID des Messquerschnitt.
   */
  private HashMap<String, MessQuerschnitt> m_mapMessQuerschnitte = new HashMap<String, MessQuerschnitt>();
  /**
   * Speicher fr die Objekte zur Verwaltung der Messquerschnittsdaten in Straenabschnitten.<br>
   * Key ist die PID des Messquerschnitt.
   */
  private HashMap<String, MessQuerschnitt> m_mapMessQuerschnitteAbschnitt = new HashMap<String, MessQuerschnitt>();
  /**
   * Liste der passivierten Fahrstreifen.
   */
  private PassivierungFahrstreifen m_passivierungFahrStreifen;
  /**
   * Definition Verkehrsstufe.
   */
  protected int m_strfallSituationStrung              = 0;
  /**
   * Definition Verkehrsstufe.
   */
  protected int m_strfallSituationKeineAussage         = 1;
  /**
   * Definition Verkehrsstufe.
   */
  protected int m_strfallSituationFreierVerkehr        = 2;
  /**
   * Definition Verkehrsstufe.
   */
  protected int m_strfallSituationLebhafterVerkehr     = 3;
  /**
   * Definition Verkehrsstufe.
   */
  protected int m_strfallSituationDichterVerkehr       = 4;
  /**
   * Definition Verkehrsstufe.
   */
  protected int m_strfallSituationZhflieenderVerkehr = 5;
  /**
   * Definition Verkehrsstufe.
   */
  protected int m_strfallSituationStockenderVerkehr    = 6;
  /**
   * Definition Verkehrsstufe.
   */
  private final int m_strfallSituationStau             = 7;
  
  /**
   * Klassifizierung Staukriterium 1.
   */
  private Staukriterium m_staukriterium1;
  /**
   * Klassifizierung Staukriterium 2.
   */
  private Staukriterium m_staukriterium2;
  /**
   * Klassifizierung Staukriterium 3.
   */
  private Staukriterium m_staukriterium3;
  /**
   * Klassifizierung Staukriterium 4.
   */
  private Staukriterium m_staukriterium4;
  /**
   * Klassifizierung Staukriterium 6.
   */
  private Staukriterium m_staukriterium6;
  
  /**
   * Liste der Ergebnisse der Staukriterien fr die Zusammenfassung.
   */
  private List<Staukriterium> m_staukriteriumListe;
  /**
   * Verkehrslicher Zustand des Staukriteriums.<br>
   * true = Staukriterium aktiv, false = Staukriterium inaktiv.
   */
  private boolean m_staukriterium1ZustandVerkehrlich = false;
  /**
   * Verkehrslicher Zustand des Staukriteriums.<br>
   * true = Staukriterium aktiv, false = Staukriterium inaktiv.
   */
  private boolean m_staukriterium2ZustandVerkehrlich = false;
  /**
   * Verkehrslicher Zustand des Staukriteriums.<br>
   * true = Staukriterium aktiv, false = Staukriterium inaktiv.
   */
  private boolean m_staukriterium3ZustandVerkehrlich = false;
  /**
   * Verkehrslicher Zustand des Staukriteriums.<br>
   * true = Staukriterium aktiv, false = Staukriterium inaktiv.
   */
  private boolean m_staukriterium6ZustandVerkehrlich = false;
  
  private final boolean m_vgKfzVerwenden = true;
	/**
	 * bergabeparameter -verknuepfungMqAlsUnd=true/false</br>
	 * Defaultwert=false
	 */
  private boolean m_verknuepfungMqAlsUnd;
	/**
	 * Attributgruppe fr den Empfang der Parameter FS-Orientiert.
	 */
  private AttributeGroup m_atgParameterFsOrientiert;
  /**
   * Parameter fr die Situationsbestimmung Staukriterium 6 FS-Orientiert.
   */  
  private ParameterStauFsOrientiert m_parameterStauFsOrientiert;
	/**
	 * Speicher fr die Objekte Anzeige, zur Bestimmung der aktuellen Rckmeldung.
	 */
  private Map<Integer, Anzeige> m_hmAnzeigen = Collections.synchronizedMap(new HashMap<Integer, Anzeige>());
  
  public enum Kriterium
  {
  	SKE,
  	SKA;
  }
  /**
   * Objekt WVZ-Inhalt 'Dunkel'.
   */
	private SystemObject m_objektWvzInhaltDunkel;
	
  /**
   * Kennung, ob die Modellerweiterung Stau Fs-Orientiert vorhanden ist.
   */
  private boolean m_modellStauFsOrientiertvorhanden;
  
	/**
	 * Konstruktor mit bernahme der Parameter.<p>
	 * Anmelden zum Empfang der bentigten Parameter am Datenverteiler.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param ue Ursacheneinheit.
	 * @param aspektVersand Aspekt fr den Versand.
	 */
	public StauUe(ClientDavInterface connection, SystemObject ue, Aspect aspektVersand, boolean verknuepfungMqAlsUnd)
	{
		super(connection, ue, aspektVersand);

		m_objekt = ue;
		m_verknuepfungMqAlsUnd = verknuepfungMqAlsUnd;
		
		m_modellStauFsOrientiertvorhanden = false;

		/**
		 * Prfen, ob die Modellerweiterung Stau FS-Orientiert vorhanxden ist.
		 */
		List<AttributeGroup> liste = ue.getType().getAttributeGroups();		
		for (AttributeGroup attributeGroup : liste)
		{			
			Collection<AttributeGroupUsage> attributeGroupUsages = attributeGroup.getAttributeGroupUsages();

			for (AttributeGroupUsage attributeGroupUsage : attributeGroupUsages)
			{
				if (attributeGroupUsage.getAttributeGroup().getPid().equals("atg.sswSbaUeStauFsOrientiert"))
				{
//					System.out.println(m_objekt + " !!!!!!!!!! Modell Stau-FS-Orientiert vorhanden !!!!!!!!");

					m_modellStauFsOrientiertvorhanden = true;
					
					break;
				}
			}
		}
		
		m_atgParameterMqZuordnung = m_dataModel.getAttributeGroup("atg.sswSbaUeMqZuordnung");
		m_atgParameterStauBeleg = m_dataModel.getAttributeGroup("atg.sswSbaUeStauBeleg");
		m_atgParameterStauGeschw = m_dataModel.getAttributeGroup("atg.sswSbaUeStauGeschw");
		m_atgParameterStauStoerfall = m_dataModel.getAttributeGroup("atg.sswSbaUeStauStrfall");
		m_atgParameterStaukriterium = m_dataModel.getAttributeGroup("atg.sswSbaUeStaukriterium");
				
		m_aspParameter = m_dataModel.getAspect("asp.parameterSoll");

		m_atgZusammenfassungStaukriterien = m_dataModel.getAttributeGroup("atg.sswSbaUeStaukriteriumZusammenfassung");
		m_atgErgebnisStaukriterien = m_dataModel.getAttributeGroup("atg.sswSbaUeStaukriteriumErgebnis");

		m_atgAnalyse = m_dataModel.getAttributeGroup("atg.sswSbaUeSituationAnalyseMq");
		m_aspAnalyse = m_dataModel.getAspect("asp.analyse");

		m_staukriterium1 = new Staukriterium();
		m_staukriterium2 = new Staukriterium();
		m_staukriterium3 = new Staukriterium();
		m_staukriterium4 = new Staukriterium();
		
		m_staukriteriumListe = new ArrayList<Staukriterium>();
		
		if (m_modellStauFsOrientiertvorhanden)
		{
			m_atgParameterFsOrientiert = m_dataModel.getAttributeGroup("atg.sswSbaUeStauFsOrientiert");
			m_staukriterium6 = new Staukriterium();
			
			m_objektWvzInhaltDunkel = bestimmeWvzInhaltDunkel();
			
			empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameterFsOrientiert, m_aspParameter);
		}
		
		// Daten zum Empfang anmelden
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameterMqZuordnung, m_aspParameter);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameterStauBeleg, m_aspParameter);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameterStauGeschw, m_aspParameter);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameterStauStoerfall, m_aspParameter);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameterStaukriterium, m_aspParameter);

		// Daten zum Versand anmelden
		quelleAnmeldenObjekt(this, m_objekt, m_atgAnalyse, m_aspAnalyse);
		quelleAnmeldenObjekt(this, m_objekt, m_atgZusammenfassungStaukriterien, m_aspAnalyse);
		quelleAnmeldenObjekt(this, m_objekt, m_atgErgebnisStaukriterien, m_aspAnalyse);
	}
	
	/**
	 * Diese Methode berechnet die aktuelle Situation der Ursacheneinheit entsprechend den
	 * empfangenen Parametern und versendet die Analysedaten ber den Datenverteiler..
	 */
	public boolean berechnen()
	{
		boolean _statusAlt = this.getStatus();
		boolean zurueck = false;
    boolean stoerungMesswerte = false;
	  boolean einschaltKriterium = false;
    boolean ausschaltKriterium = false;
    int ausschaltZaehlerMq;
    int einschaltZaehlerMq;
    
    int mwSk1Ok = 0;
    int mwSk2Ok = 0;
    int mwSk3Ok = 0;
    int mwSk4Ok = 0;
    int mwSk6Ok = 0;
    
    SituationAnalyseMq analyse = new SituationAnalyseMq();
    StaukriteriumZusammenfassung skZus = null;
    
    m_staukriteriumListe.clear();
    
    ausschaltZaehlerMq = 0;
    einschaltZaehlerMq = 0;
    
    Object[] keys = m_mapMessQuerschnitte.keySet().toArray();
    for (int i = 0; i < keys.length; i++)
		{
			String pidMq = (String)keys[i];
			
			MessQuerschnitt mq = m_mapMessQuerschnitte.get(pidMq);
			if (mq != null)
			{
				boolean einschaltKriteriumMq = false;
				boolean ausschaltKriteriumMq = false;
				
				/**
				 * Staukriterizum 1
				 */
				boolean[] staukriterium1Ein = new boolean[1];
				boolean[] staukriterium1Aus = new boolean[1];
				
				mwSk1Ok = (!staukriterium1Berechnen(mq, staukriterium1Ein, staukriterium1Aus, _statusAlt)) ? (mwSk1Ok+1) : mwSk1Ok;
				System.out.println("Staukriterium 1 Messwerte ok " + mwSk1Ok + " einKrit " + staukriterium1Ein[0] + " ausKrit " + staukriterium1Aus[0]);
				
//				stoerungMesswerte |= staukriterium1Berechnen(mq, staukriterium1Ein, staukriterium1Aus);
//				System.out.println("Staukriterium 1 Messwerte gestrt " + stoerungMesswerte + " einKrit " + staukriterium1Ein[0] + " ausKrit " + staukriterium1Aus[0]);

				/**
				 * Staukriterium 2
				 */
				boolean[] staukriterium2Ein = new boolean[1];
				boolean[] staukriterium2Aus = new boolean[1];

				mwSk2Ok = (!staukriterium2Berechnen(mq, staukriterium2Ein, staukriterium2Aus, _statusAlt)) ? (mwSk2Ok+1) : mwSk2Ok;
				System.out.println("Staukriterium 2 Messwerte ok " + mwSk2Ok + " einKrit " + staukriterium2Ein[0] + " ausKrit " + staukriterium2Aus[0]);

//				stoerungMesswerte |= staukriterium2Berechnen(mq, staukriterium2Ein, staukriterium2Aus);
//				System.out.println("Staukriterium 2 Messwerte gestrt " + stoerungMesswerte + " einKrit " + staukriterium2Ein[0] + " ausKrit " + staukriterium2Aus[0]);
				
				/**
				 * Staukriterium 3
				 */
				boolean[] staukriterium3Ein = new boolean[1];
				boolean[] staukriterium3Aus = new boolean[1];

				mwSk3Ok = (!staukriterium3Berechnen(mq, staukriterium3Ein, staukriterium3Aus, _statusAlt)) ? (mwSk3Ok+1) : mwSk3Ok;
				System.out.println("Staukriterium 3 Messwerte ok " + mwSk3Ok + " einKrit " + staukriterium3Ein[0] + " ausKrit " + staukriterium3Aus[0]);

//				stoerungMesswerte |= staukriterium3Berechnen(mq, staukriterium3Ein, staukriterium3Aus);				
//				System.out.println("Staukriterium 3 Messwerte gestrt " + stoerungMesswerte + " einKrit " + staukriterium3Ein[0] + " ausKrit " + staukriterium3Aus[0]);

				/**
				 * Staukriterium 4
				 */
				boolean[] staukriterium4Ein = new boolean[1];
				boolean[] staukriterium4Aus = new boolean[1];

				mwSk4Ok = (!staukriterium4Berechnen(mq, staukriterium4Ein, staukriterium4Aus, _statusAlt)) ? (mwSk4Ok+1) : mwSk4Ok;
				System.out.println("Staukriterium 4 Messwerte ok " + mwSk4Ok + " einKrit " + staukriterium4Ein[0] + " ausKrit " + staukriterium4Aus[0]);

//				stoerungMesswerte |= staukriterium4Berechnen(mq, staukriterium4Ein, staukriterium4Aus);				
//				System.out.println("Staukriterium 4 Messwerte gestrt " + stoerungMesswerte + " einKrit " + staukriterium3Ein[0] + " ausKrit " + staukriterium3Aus[0]);

				/**
				 * Staukriterium 6
				 */
				boolean[] staukriterium6Ein = new boolean[1];
				boolean[] staukriterium6Aus = new boolean[1];

				if (m_modellStauFsOrientiertvorhanden)
				{
					mwSk6Ok = (!staukriterium6Berechnen(mq, staukriterium6Ein, staukriterium6Aus, _statusAlt)) ? (mwSk6Ok+1) : mwSk6Ok;
					System.out.println("Staukriterium 6 Messwerte ok " + mwSk6Ok + " einKrit " + staukriterium6Ein[0] + " ausKrit " + staukriterium6Aus[0]);
				}
				else
				{
					staukriterium6Ein[0] = false;
					staukriterium6Aus[0] = true;
				}
				
				/**
				 * Ergebnis
				 */

				einschaltKriteriumMq = staukriterium1Ein[0] | staukriterium2Ein[0] | staukriterium3Ein[0] | staukriterium4Ein[0] | staukriterium6Ein[0];
				ausschaltKriteriumMq = staukriterium1Aus[0] && staukriterium2Aus[0] && staukriterium3Aus[0] && staukriterium4Aus[0] && staukriterium6Aus[0];

				if (!m_verknuepfungMqAlsUnd)
				{
					einschaltKriterium |= einschaltKriteriumMq;
					
					if (ausschaltKriteriumMq)
					{
						ausschaltZaehlerMq++;
					}				
				}
				else
				{
					if (einschaltKriteriumMq)
					{
						einschaltZaehlerMq++;
					}
					
					ausschaltKriterium |= ausschaltKriteriumMq;
				}
				
				analyse.addAnalyseMq(mq.getObjekt(), einschaltKriteriumMq);

				System.out.println(m_objekt.getName() + " MQ='" + mq.getPid() + "' Analyse=" + analyse);
			}
		}

		if (!m_verknuepfungMqAlsUnd)
		{
	    if (ausschaltZaehlerMq == keys.length)
	    {
	    	ausschaltKriterium = true;
	    }
		}
		else
		{
			if (einschaltZaehlerMq == keys.length)
			{
				einschaltKriterium = true;
			}
		}
		
		analyse.setEinschaltkriterium(einschaltKriterium);
		analyse.setAusschaltkriterium(ausschaltKriterium);

    // Analysedaten versenden.
    sendeAnalyseDaten(analyse);

		System.out.println(m_objekt.getName() + " Messwerte gestrt=" + stoerungMesswerte + " einKrit=" + einschaltKriterium + " ausKrit=" + ausschaltKriterium + " VerknpfungMqAlsUnd=" + m_verknuepfungMqAlsUnd);
		
		zurueck =	bestimmeKriteriumMq(einschaltKriterium, ausschaltKriterium, _statusAlt);
								
    // Ergebnisse der Staukriterien versenden
    sendeStaukriteriumErgebnis();
    
    skZus = StaukriteriumKlassifizierung.staukriteriumZusammenfassung(m_staukriteriumListe);

		System.out.println(m_objekt.getName() + " Zusammenfassung Staukriterien " + skZus);

    // Zusammenfassung der Staukriterien versenden
    sendeStaukriteriumZusammenfassung(skZus);

    // Keine Messwerte vorhanden, Schaltgund deaktivieren 
    if ((mwSk1Ok == 0) && (mwSk2Ok == 0) && (mwSk3Ok == 0) && (mwSk4Ok == 0) && (mwSk6Ok == 0))
    	zurueck = false;

    System.out.println(m_objekt.getName() + " Aktueller Status=" + zurueck);
    
    return zurueck;
	}
 
	/**
	 * Diese Methode berechnet den Status Staukriterium 1 (Belegung). 
	 *  
	 * @param mq MessQuerschnitt.
	 * @param einkrit Variable zum Speichen des Einschaltkriterium.
	 * @param auskrit Variable zum Speichen des Ausschaltkriterium.
	 * @param statusUeAlt Status der UE (t-1).
	 * @return true wenn eine Strung der Messwerte ermittelt wurde, ansonsten false.
	 */
	private boolean staukriterium1Berechnen(MessQuerschnitt mq, boolean[] einkrit, boolean[] auskrit, boolean statusUeAlt)
	{
		boolean stoerungMesswerteFs = true;
		boolean stoerungMesswerteMq = true;
		auskrit[0] = true;

		float guete = 0.0F;
    float zStau = (float)0.0;
    float zNichtStau = (float)1.0;
    float vStau = (float)0.0;
    float vNichtStau = (float)1.0;

		Staukriterium sk = new Staukriterium();
		
		if (m_parameterStauBeleg == null)
		{
//			System.out.println(m_objekt.getPid() + " Noch keine Parameter erhalten. Abbruch!");
			m_debug.fine(m_objekt.getPid() + " Noch keine Parameter erhalten. Abbruch!");

			return false;
		}

		// Prfen, auf Deaktivierung des Staukriteriums 
		if (m_parameterStaukriterium != null)
		{
			if (m_parameterStaukriterium.getStaukriterium1().getStau().getVertrauenswrdigkeit() == 0)
			{
				System.out.println(m_objekt.getPid() + " Staukriterium 1 (Belegung) ist deaktiviert!");
				m_debug.fine(m_objekt.getPid() + " Staukriterium 1 (Belegung) ist deaktiviert!");
				
				return false;
			}
		}			
		
		int anzFs = bestimmeAnzFahrstreifenAktuell(mq);
		
    System.out.println("Parameter Stau Belegung -> " + mq.getPid() +  " anzFs " + anzFs + " " + m_parameterStauBeleg);
    m_debug.fine("Parameter Stau Belegung -> " + mq.getPid() + " anzFs " + anzFs + " " + m_parameterStauBeleg);
    
		VerkehrsDatenKurzZeitTrendExtraPolationMq prognoseDaten = mq.getVerkehrsDatenKurzZeitTrendExtraPolationMq();
						
//		System.out.println("VerkehrsDaten Prognose " + mq.getPid() + " " + prognoseDaten);
		
		if (prognoseDaten == null)
			return stoerungMesswerteMq;
				
		int _vKfzP = prognoseDaten.getVKfzP();
		float gueteVkfzP = prognoseDaten.getVKfzPGte();
		
		List<SystemObject> _fahrstreifen = mq.getFahrstreifen();
		
		float gueteFs = 0.0F;
		float gueteFsSumme = 0.0F;
		int anzahlFs = 0;
		
		// Loop ber alle Fahrstreifen des MQ's
		for (int i = 0; i < _fahrstreifen.size(); i++)
		{
			SystemObject fs = _fahrstreifen.get(i);
			
			// Fahrstreifen passiviert ?
			if (isFahrstreifenPassiviert(fs))
			{
				System.out.println("Fahrstreifen passiviert! " + fs.getPid());
				m_debug.fine("Fahrstreifen passiviert! " + fs.getPid());
				
				continue;
			}
			
			int _vStauEin = m_parameterStauBeleg.getParameterFahrstreifen(i).getVStauEin();
	    int _bStauEin = m_parameterStauBeleg.getParameterFahrstreifen(i).getBStauEin();
	    int _bStauAus = m_parameterStauBeleg.getParameterFahrstreifen(i).getBStauAus();
	    
//	    System.out.println(i + " _vStauEin " + _vStauEin + " _bStauEin " + _bStauEin + " _bStauAus " + _bStauAus);
	    
			VerkehrsDatenKurzZeitFs datenFs = mq.getVerkehrsDatenKurzZeitFs(fs.getPid());
			if (datenFs == null)
				continue;
			
//			System.out.println("VerkehrsDaten FS intervall " + fs.getPid() + " " + datenFs);

			// keine Daten vohanden ?
			if ((!datenFs.hasDatenErhalten()) || (!prognoseDaten.hasDatenErhalten()))
				continue;
						
			int _bFs = datenFs.getB();
			gueteFsSumme += datenFs.getBGte();
			anzahlFs++;
			
			
			if ((_bFs >= 0) && (_vKfzP >= 0))
			{
				stoerungMesswerteFs = false;
				stoerungMesswerteMq = false;
				
				if ((_bFs > _bStauEin) && (_vKfzP <= _vStauEin))
				{
					einkrit[0] = true;
				}
			}
				
			if (_bFs >= 0)
			{
				if (_bFs >= _bStauAus)
				{
					auskrit[0] = false;
				}
			}
		}
		
		if (anzahlFs > 0)
		{
			gueteFs = gueteFsSumme / anzahlFs;
		}
		
		guete = gueteVkfzP * gueteFs;

		if (!stoerungMesswerteFs && !stoerungMesswerteMq)
		{
			if (m_parameterStaukriterium != null)
			{
				vNichtStau = m_parameterStaukriterium.getStaukriterium1().getNichtStau().getVertrauenswrdigkeit();
				vStau = m_parameterStaukriterium.getStaukriterium1().getStau().getVertrauenswrdigkeit();
				
				zNichtStau = m_parameterStaukriterium.getStaukriterium1().getNichtStau().getZugehrigkeit();
				zStau = m_parameterStaukriterium.getStaukriterium1().getStau().getZugehrigkeit();
			}			
			
			zStau *= StaukriteriumKlassifizierung.zugehoerigkeitStau(einkrit[0], auskrit[0], m_staukriterium1ZustandVerkehrlich);
			zNichtStau *= StaukriteriumKlassifizierung.zugehoerigkeitNichtStau(einkrit[0], auskrit[0], m_staukriterium1ZustandVerkehrlich);

			m_staukriterium1ZustandVerkehrlich = bestimmeZustandVerkehrlich(einkrit[0], auskrit[0], m_staukriterium1ZustandVerkehrlich);
			boolean zustandVerkehrlich = bestimmeZustandVerkehrlich(einkrit[0], auskrit[0], m_staukriterium1ZustandVerkehrlich);

			m_staukriterium1.setPid(mq.getPid());
			m_staukriterium1.setGteNichtStau(guete);
			m_staukriterium1.setGteStau(guete);
			m_staukriterium1.setVertrauenswrdigkeitNichtStau(vNichtStau);
			m_staukriterium1.setVertrauenswrdigkeitStau(vStau);
			m_staukriterium1.setZugehrigkeitNichtStau(zNichtStau);
			m_staukriterium1.setZugehrigkeitStau(zStau);
			m_staukriterium1.setZustandVerkehrlich(zustandVerkehrlich);
			
			sk.setPid(mq.getPid());
			sk.setGteNichtStau(guete);
			sk.setGteStau(guete);
			sk.setVertrauenswrdigkeitNichtStau(vNichtStau);
			sk.setVertrauenswrdigkeitStau(vStau);
			sk.setZugehrigkeitNichtStau(zNichtStau);
			sk.setZugehrigkeitStau(zStau);
			sk.setZustandVerkehrlich(zustandVerkehrlich);
		}
		else
		{
			m_staukriterium1.setPid(mq.getPid());
			m_staukriterium1.init();
			
			sk.setPid(mq.getPid());
			sk.init();
			
			m_staukriterium1ZustandVerkehrlich = false;
		}

		if (sk.getGteNichtStau() >= m_parameterStauBeleg.getGuete())
			m_staukriteriumListe.add(sk);
		else
		{
			einkrit[0] = false;
			auskrit[0] = true;

			m_staukriterium1ZustandVerkehrlich = false;
			
			System.out.println(mq.getPid() + " Staukriterium 1 Grenzwert Gte (" + m_parameterStauBeleg.getGuete() + ") nicht erreicht");
		}

		System.out.println(mq.getPid() + " Staukriterium 1 -> " + sk);
							
		return stoerungMesswerteFs | stoerungMesswerteMq;
	}

	/**
	 * Diese Methode berechnet den Status Staukriterium 2 (Prognosegeschwindigkeit). 
	 *  
	 * @param mq MessQuerschnitt.
	 * @param einkrit Variable zum Speichen des Einschaltkriterium.
	 * @param auskrit Variable zum Speichen des Ausschaltkriterium.
	 * @param statusUeAlt Status der UE (t-1).
	 * @return true wenn eine Strung der Messwerte ermittelt wurde, ansonsten false.
	 */
	private boolean staukriterium2Berechnen(MessQuerschnitt mq, boolean[] einkrit, boolean[] auskrit, boolean statusUeAlt)
	{
		boolean stoerungMesswerteMq = true;
		auskrit[0] = true;

		float guete = 0.0F;
    float zStau = (float)0.0;
    float zNichtStau = (float)1.0;
    float vStau = (float)0.0;
    float vNichtStau = (float)1.0;

		Staukriterium sk = new Staukriterium();

		if (m_parameterStauGeschw == null)
		{
//			System.out.println(m_objekt.getPid() + " Noch keine Parameter erhalten. Abbruch!");
			m_debug.fine(m_objekt.getPid() + " Noch keine Parameter erhalten. Abbruch!");

			return false;
		}

		// Prfen, auf Deaktivierung des Staukriteriums 
		if (m_parameterStaukriterium != null)
		{
			if (m_parameterStaukriterium.getStaukriterium2().getStau().getVertrauenswrdigkeit() == 0)
			{
				System.out.println(m_objekt.getPid() + " Staukriterium 2 (Prognosegeschwindigkeit) ist deaktiviert!");
				m_debug.fine(m_objekt.getPid() + " Staukriterium 2 (Prognosegeschwindigkeit) ist deaktiviert!");
				
				return false;
			}
		}			
		
		int anzFs = bestimmeAnzFahrstreifenAktuell(mq);
		ParameterStaugeschwFahrstreifen parameter = m_parameterStauGeschw.getParameterFahrstreifen(anzFs);
		
    System.out.println("Parameter Stau Geschwindigkeit -> " + mq.getPid() +  " anzFs " + anzFs + " " + parameter);
    m_debug.fine("Parameter Stau Geschwindigkeit -> " + mq.getPid() + " anzFs " + anzFs + " " + parameter);

    if (parameter == null)
    	return false;
    
    
    int _vStauEin = parameter.getVStauEin();
    int _vStauAus = parameter.getVStauAus();
    int _vDiffStau = parameter.getVDiffStau();
    int _qKfzStau = parameter.getQKfzStau();
    
		VerkehrsDatenKurzZeitTrendExtraPolationMq prognoseDaten = mq.getVerkehrsDatenKurzZeitTrendExtraPolationMq();
		VerkehrsDatenKurzZeitMq kurzZeitDaten = mq.getVerkehrsDatenKurzZeitMq();
						
		if ((prognoseDaten == null) || (kurzZeitDaten == null))
			return stoerungMesswerteMq;
				
		if ((prognoseDaten.hasDatenErhalten()) && (kurzZeitDaten.hasDatenErhalten()))
		{
			stoerungMesswerteMq = false;

			int _vKfz = -1;
			float gueteVkfz = -1;

			if (m_vgKfzVerwenden)
			{
//				System.out.println("Verkehrsdaten Kurzzeit " + mq.getObjekt().getName() + " " + kurzZeitDaten);

				_vKfz = kurzZeitDaten.getVgKfz();
				gueteVkfz = kurzZeitDaten.getVgKfzGte();				
			}
			else
			{
//				System.out.println("VerkehrsDaten Prognose " + mq.getObjekt().getName() + " " + prognoseDaten);

				_vKfz = prognoseDaten.getVKfzP();
				gueteVkfz = prognoseDaten.getVKfzPGte();				
			}

			int _vLkwP = prognoseDaten.getVLkwP();
			int _vPkwP = prognoseDaten.getVPkwP();
			
			float gueteVlkw = prognoseDaten.getVLkwPGte();
			float gueteVpkw = prognoseDaten.getVPkwPGte();
			
			guete = gueteVkfz * gueteVlkw * gueteVpkw;
			
			int _qKfz = kurzZeitDaten.getQKfz();
			int _qLkw = kurzZeitDaten.getQLkw();
			int _qPkw = kurzZeitDaten.getQPkw();

			int vDiff = -1;
			if ((_qPkw > 0) && (_qLkw > 0))
			{
				if ((_vPkwP > 0) && (_vLkwP > 0))
				{
					vDiff = Math.abs(_vPkwP - _vLkwP);
				}
			}
			
			if ((_vKfz >= 0) && (_qKfz >= 0))
			{			
				auskrit[0] = false;

				// Einschaltkriterium
				if (vDiff > 0)
				{
					if ((_vKfz <= _vStauEin) && (_qKfz > _qKfzStau) && (vDiff <= _vDiffStau))
						einkrit[0] = true;
				}
				else
				{
					if ((_vKfz <= _vStauEin) && (_qKfz > _qKfzStau))
						einkrit[0] = true;				
				}

				// Ausschaltkriterium
				if (_vKfz >= _vStauAus)
					auskrit[0] = true;
			}

			if (m_parameterStaukriterium != null)
			{
				vNichtStau = m_parameterStaukriterium.getStaukriterium2().getNichtStau().getVertrauenswrdigkeit();
				vStau = m_parameterStaukriterium.getStaukriterium2().getStau().getVertrauenswrdigkeit();
				
				zNichtStau = m_parameterStaukriterium.getStaukriterium2().getNichtStau().getZugehrigkeit();
				zStau = m_parameterStaukriterium.getStaukriterium2().getStau().getZugehrigkeit();
			}
			
			zStau *= StaukriteriumKlassifizierung.zugehoerigkeitStau(einkrit[0], auskrit[0], m_staukriterium2ZustandVerkehrlich);
			zNichtStau *= StaukriteriumKlassifizierung.zugehoerigkeitNichtStau(einkrit[0], auskrit[0], m_staukriterium2ZustandVerkehrlich);
			
			m_staukriterium2ZustandVerkehrlich = bestimmeZustandVerkehrlich(einkrit[0], auskrit[0], m_staukriterium2ZustandVerkehrlich);
			boolean zustandVerkehrlich = bestimmeZustandVerkehrlich(einkrit[0], auskrit[0], m_staukriterium2ZustandVerkehrlich);

			m_staukriterium2.setPid(mq.getPid());
			m_staukriterium2.setGteNichtStau(guete);
			m_staukriterium2.setGteStau(guete);
			m_staukriterium2.setVertrauenswrdigkeitNichtStau(vNichtStau);
			m_staukriterium2.setVertrauenswrdigkeitStau(vStau);
			m_staukriterium2.setZugehrigkeitNichtStau(zNichtStau);
			m_staukriterium2.setZugehrigkeitStau(zStau);
			m_staukriterium2.setZustandVerkehrlich(zustandVerkehrlich);
			
			sk.setPid(mq.getPid());
			sk.setGteNichtStau(guete);
			sk.setGteStau(guete);
			sk.setVertrauenswrdigkeitNichtStau(vNichtStau);
			sk.setVertrauenswrdigkeitStau(vStau);
			sk.setZugehrigkeitNichtStau(zNichtStau);
			sk.setZugehrigkeitStau(zStau);
			sk.setZustandVerkehrlich(zustandVerkehrlich);
		}
		else
		{
			m_staukriterium2.setPid(mq.getPid());
			m_staukriterium2.init();

			sk.setPid(mq.getPid());
			sk.init();
			
			m_staukriterium2ZustandVerkehrlich = false;
		}
		
		if (sk.getGteNichtStau() >= m_parameterStauGeschw.getGuete())
			m_staukriteriumListe.add(sk);
		else
		{
			einkrit[0] = false;
			auskrit[0] = true;

			m_staukriterium2ZustandVerkehrlich = false;
			
			System.out.println(mq.getPid() + " Staukriterium 2 Grenzwert Gte (" + m_parameterStauGeschw.getGuete() + ") nicht erreicht");
		}

		System.out.println(mq.getPid() + " Staukriterium 2 -> " + sk);
		
		return stoerungMesswerteMq;
	}

	/**
	 * Diese Methode berechnet den Status Staukriterium 3 (abschnittsbezogene Strfallerkennung). 
	 *  
	 * @param mq MessQuerschnitt.
	 * @param einkrit Variable zum Speichen des Einschaltkriterium.
	 * @param auskrit Variable zum Speichen des Ausschaltkriterium.
	 * @param statusUeAlt Status der UE (t-1).
	 * @return true wenn eine Strung der Messwerte ermittelt wurde, ansonsten false.
	 */
	private boolean staukriterium3Berechnen(MessQuerschnitt mq, boolean[] einkrit, boolean[] auskrit, boolean statusUeAlt)
	{
		boolean stoerungMesswerteMq = true;
		auskrit[0] = true;

		float guete = 0.0F;
    float zStau = (float)0.0;
    float zNichtStau = (float)1.0;
    float vStau = (float)0.0;
    float vNichtStau = (float)1.0;

		Staukriterium sk = new Staukriterium();

		if (m_parameterStauStoerfall == null)
		{
//			System.out.println(m_objekt.getPid() + " Noch keine Parameter erhalten. Abbruch!");
			m_debug.fine(m_objekt.getPid() + " Noch keine Parameter erhalten. Abbruch!");

			return false;
		}

		// Prfen, auf Deaktivierung des Staukriteriums 
		if (m_parameterStaukriterium != null)
		{
			if (m_parameterStaukriterium.getStaukriterium3().getStau().getVertrauenswrdigkeit() == 0)
			{
				System.out.println(m_objekt.getPid() + " Staukriterium 3 (abschnittsbezogene Strfallerkennung) ist deaktiviert!");
				m_debug.fine(m_objekt.getPid() + " Staukriterium 3 (abschnittsbezogene Strfallerkennung) ist deaktiviert!");
				
				return false;
			}
		}			
		
		int anzFs = bestimmeAnzFahrstreifenAktuell(mq);
		ParameterStauStoerfallFahrstreifen parameter = m_parameterStauStoerfall.getParameterFahrstreifen(anzFs);
		
		if (parameter == null)
			return false;

		MessQuerschnitt mq2 = null;
		
		SystemObject mqAusfahrend = m_parameterStauStoerfall.getMessQuerschnittAusfahrend(mq.getPid());
		if (mqAusfahrend != null)
		{
			mq2 = m_mapMessQuerschnitteAbschnitt.get(mqAusfahrend.getPid());			
		}
		
		if (mq2 == null)
		{
			System.out.println(mq.getPid() + " MQ nicht als einfahrend in einem Straenabschnitt vorhanden. Keine Berechnung!");
			return false;				
		}
		
		System.out.println("Starenabschnitt -> MQ einfahrend " + mq.getPid() + " MQ ausfahrend " + mq2.getPid());
		System.out.println("Parameter Stau Strfall -> " + mq.getPid() +  " anzFs " + anzFs + " " + parameter);
    m_debug.fine("Parameter Stau Strfall -> " + mq.getPid() + " anzFs " + anzFs + " " + parameter);

    // Daten holen
		VerkehrsDatenKurzZeitTrendExtraPolationMq prognoseDaten = mq.getVerkehrsDatenKurzZeitTrendExtraPolationMq();
		VerkehrsDatenKurzZeitTrendExtraPolationMq prognoseDaten2 = mq2.getVerkehrsDatenKurzZeitTrendExtraPolationMq();
		VerkehrsDatenKurzZeitMq kurzZeitDaten = mq.getVerkehrsDatenKurzZeitMq();

//		System.out.println("VerkehrsDaten Prognose MQ 1" + mq.getPid() + " " + prognoseDaten);
//		System.out.println("VerkehrsDaten Prognose MQ 2" + mq2.getPid() + " " + prognoseDaten2);
//		System.out.println("VerkehrsDaten Kurzzeit MQ 1" + mq.getPid() + " " + kurzZeitDaten);
		
		if ((prognoseDaten == null) || (prognoseDaten2 == null) || (kurzZeitDaten == null))
		{
			auskrit[0] = true;

			return stoerungMesswerteMq;
		}
		
		if ((prognoseDaten.hasDatenErhalten()) && (prognoseDaten2.hasDatenErhalten()))
		{
			stoerungMesswerteMq = false;

			// vkDiff berechnen
			float vkDiff = berechneVkDiff(prognoseDaten, prognoseDaten2, parameter);
			guete = berechneGueteVkDiff(prognoseDaten, prognoseDaten2);
			
			// Parameter
			float _vkGrenzEin = parameter.getVkGrenzEin();
			float _vkGrenzAus = parameter.getVkGrenzAus();
			int _qKfzDiffEin = parameter.getQKfzDiffEin();
			int _qKfzDiffAus = parameter.getQKfzDiffAus();

			int qKfz = kurzZeitDaten.getQKfz();

			System.out.println(mq.getPid() + " vkDiff -> " + vkDiff + " QKfz " + qKfz + " Gte " + guete);
						
			if (qKfz >= 0)
			{
				auskrit[0] = false;

				if ((Math.abs(vkDiff) >= _vkGrenzEin) && (qKfz >= _qKfzDiffEin))
				{
					einkrit[0] = true;
				}

				if ((Math.abs(vkDiff) < _vkGrenzAus) || (qKfz < _qKfzDiffAus))
				{
					auskrit[0] = true;
				}			
			}
						
			if (m_parameterStaukriterium != null)
			{
				vNichtStau = m_parameterStaukriterium.getStaukriterium3().getNichtStau().getVertrauenswrdigkeit();
				vStau = m_parameterStaukriterium.getStaukriterium3().getStau().getVertrauenswrdigkeit();
				
				zNichtStau = m_parameterStaukriterium.getStaukriterium3().getNichtStau().getZugehrigkeit();
				zStau = m_parameterStaukriterium.getStaukriterium3().getStau().getZugehrigkeit();
			}

			zStau *= StaukriteriumKlassifizierung.zugehoerigkeitStau(einkrit[0], auskrit[0], m_staukriterium3ZustandVerkehrlich);
			zNichtStau *= StaukriteriumKlassifizierung.zugehoerigkeitNichtStau(einkrit[0], auskrit[0], m_staukriterium3ZustandVerkehrlich);
			
			m_staukriterium3ZustandVerkehrlich = bestimmeZustandVerkehrlich(einkrit[0], auskrit[0], m_staukriterium3ZustandVerkehrlich);
			boolean zustandVerkehrlich = bestimmeZustandVerkehrlich(einkrit[0], auskrit[0], m_staukriterium3ZustandVerkehrlich);

			m_staukriterium3.setPid(mq.getPid());
			m_staukriterium3.setGteNichtStau(guete);
			m_staukriterium3.setGteStau(guete);
			m_staukriterium3.setVertrauenswrdigkeitNichtStau(vNichtStau);
			m_staukriterium3.setVertrauenswrdigkeitStau(vStau);
			m_staukriterium3.setZugehrigkeitNichtStau(zNichtStau);
			m_staukriterium3.setZugehrigkeitStau(zStau);
			m_staukriterium3.setZustandVerkehrlich(zustandVerkehrlich);
			
			sk.setPid(mq.getPid());
			sk.setGteNichtStau(guete);
			sk.setGteStau(guete);
			sk.setVertrauenswrdigkeitNichtStau(vNichtStau);
			sk.setVertrauenswrdigkeitStau(vStau);
			sk.setZugehrigkeitNichtStau(zNichtStau);
			sk.setZugehrigkeitStau(zStau);
			sk.setZustandVerkehrlich(zustandVerkehrlich);
		}
		else
		{
			m_staukriterium3.setPid(mq.getPid());
			m_staukriterium3.init();

			sk.setPid(mq.getPid());
			sk.init();
			
			m_staukriterium3ZustandVerkehrlich = false;
		}

		if (sk.getGteNichtStau() >= m_parameterStauStoerfall.getGuete())
			m_staukriteriumListe.add(sk);
		else
		{
			einkrit[0] = false;
			auskrit[0] = true;

			m_staukriterium3ZustandVerkehrlich = false;
			
			System.out.println(mq.getPid() + " Staukriterium 3 Grenzwert Gte (" + m_parameterStauStoerfall.getGuete() + ") nicht erreicht");
		}

		System.out.println(mq.getPid() + " Staukriterium 3 -> " + sk);
					
		return stoerungMesswerteMq;
	}

	/**
	 * Diese Methode berechnet den Gteindex fr die vkDiff Berechnung anhand der verwendeten Messwerte und
	 * deren Gten. Die Berechnung der Gte erfolgt ber die arithmetische Mittelwertbildung.
	 * 
	 * @param prognoseDaten Prognosedaten Messquerschnitt 1.
	 * @param prognoseDaten2 Prognosedaten Messquerschnitt 2.
	 * @return Gteindex der vkDiff Berechnung.
	 */
	private float berechneGueteVkDiff(VerkehrsDatenKurzZeitTrendExtraPolationMq prognoseDaten, VerkehrsDatenKurzZeitTrendExtraPolationMq prognoseDaten2)
	{
		float _gueteVKfzP1 = prognoseDaten.getVKfzPGte();
		float _gueteDP1 = prognoseDaten.getKKfzPGte();

		float _gueteVKfzP2 = prognoseDaten2.getVKfzPGte();
		float _gueteDP2 = prognoseDaten2.getKKfzPGte();

		float gueteT1 = (_gueteVKfzP1 + _gueteDP1) / 2;
		float gueteT2 = (_gueteVKfzP2 + _gueteDP2) / 2;
		
		return ((gueteT1 + gueteT2) / 2);
	}

	/**
	 * Diese Methode berechnet den Status Staukriterium 4 (Verkehrsstufe nach MARZ). 
	 *  
	 * @param mq MessQuerschnitt.
	 * @param einkrit Variable zum Speichen des Einschaltkriterium.
	 * @param auskrit Variable zum Speichen des Ausschaltkriterium.
	 * @param statusUeAlt Status der UE (t-1).
	 * @return true wenn eine Strung der Messwerte ermittelt wurde, ansonsten false.
	 */
	private boolean staukriterium4Berechnen(MessQuerschnitt mq, boolean[] einkrit, boolean[] auskrit, boolean statusUeAlt)
	{
		boolean stoerungMesswerteMq = true;
		auskrit[0] = true;

		float guete = 0.0F;
    float zStau = (float)0.0;
    float zNichtStau = (float)1.0;
    float vStau = (float)0.0;
    float vNichtStau = (float)1.0;

    Staukriterium sk = new Staukriterium();

    
    // Daten holen
		StoerfallZustand stoerfallZustand = mq.getStoerfallZustand();

		System.out.println("Strfallzustand MQ " + mq.getPid() + " " + stoerfallZustand);
		
		if (stoerfallZustand == null)
		{
			return stoerungMesswerteMq;
		}
		
		if (stoerfallZustand.hasDatenErhalten())
		{
			stoerungMesswerteMq = false;
			int vst = stoerfallZustand.getSituation();
			guete = stoerfallZustand.getGuete();
			
			System.out.println(mq.getPid() + " vst -> " + vst);
									
			if (m_parameterStaukriterium != null)
			{
				vNichtStau = m_parameterStaukriterium.getStaukriterium4().getNichtStau().getVertrauenswrdigkeit();
				vStau = m_parameterStaukriterium.getStaukriterium4().getStau().getVertrauenswrdigkeit();
				
	  		if (vst == m_strfallSituationFreierVerkehr)
	  		{
	  			zNichtStau = m_parameterStaukriterium.getStaukriterium4().getVerkehrsStufeNichtStau().getFreierVerkehr();
	  			zStau = m_parameterStaukriterium.getStaukriterium4().getVerkehrsStufeStau().getFreierVerkehr();
	  		}
	  		if (vst == m_strfallSituationDichterVerkehr)
	  		{
	  			zNichtStau = m_parameterStaukriterium.getStaukriterium4().getVerkehrsStufeNichtStau().getDichterVerkehr();
	  			zStau = m_parameterStaukriterium.getStaukriterium4().getVerkehrsStufeStau().getDichterVerkehr();
	  		}
	
	  		if (vst == m_strfallSituationZhflieenderVerkehr)
	  		{
	  			zNichtStau = m_parameterStaukriterium.getStaukriterium4().getVerkehrsStufeNichtStau().getZhfliessend();
	  			zStau = m_parameterStaukriterium.getStaukriterium4().getVerkehrsStufeStau().getZhfliessend();
	  		}
	
	  		if (vst == m_strfallSituationStau)
	  		{
	  			zNichtStau = m_parameterStaukriterium.getStaukriterium4().getVerkehrsStufeNichtStau().getStau();
	  			zStau = m_parameterStaukriterium.getStaukriterium4().getVerkehrsStufeStau().getStau();
	  		}

	  		zStau *= m_parameterStaukriterium.getStaukriterium4().getStau().getZugehrigkeit();
	  		zNichtStau *= m_parameterStaukriterium.getStaukriterium4().getNichtStau().getZugehrigkeit();
			}
						
			m_staukriterium4.setPid(mq.getPid());
			m_staukriterium4.setGteNichtStau(guete);
			m_staukriterium4.setGteStau(guete);
			m_staukriterium4.setVertrauenswrdigkeitNichtStau(vNichtStau);
			m_staukriterium4.setVertrauenswrdigkeitStau(vStau);
			m_staukriterium4.setZugehrigkeitNichtStau(zNichtStau);
			m_staukriterium4.setZugehrigkeitStau(zStau);

			sk.setPid(mq.getPid());
			sk.setGteNichtStau(guete);
			sk.setGteStau(guete);
			sk.setVertrauenswrdigkeitNichtStau(vNichtStau);
			sk.setVertrauenswrdigkeitStau(vStau);
			sk.setZugehrigkeitNichtStau(zNichtStau);
			sk.setZugehrigkeitStau(zStau);

			/**
			 * Staukriterium bestimmen
			 */
			
			if (vst >= 0)
			{
				auskrit[0] = false;

				if ((vst == m_strfallSituationStau) && (vStau > 0))
				{
					einkrit[0] = true;
				}
				else
				{
					auskrit[0] = true;
				}			
			}

		}
		else
		{
			m_staukriterium4.setPid(mq.getPid());
			m_staukriterium4.init();

			sk.setPid(mq.getPid());
			sk.init();
		}
		
		m_staukriteriumListe.add(sk);
		
		System.out.println("Staukriterium 4 Gte -> " + sk);
					
		return stoerungMesswerteMq;
	}

	/**
	 * Diese Methode berechnet den Status Staukriterium 6 (FS-orientiert).
	 *  
	 * @param mq MessQuerschnitt.
	 * @param einkrit Variable zum Speichen des Einschaltkriterium.
	 * @param auskrit Variable zum Speichen des Ausschaltkriterium.
	 * @param statusUeAlt Status der UE (t-1).
	 * @return true wenn eine Strung der Messwerte ermittelt wurde, ansonsten false.
	 */
	private boolean staukriterium6Berechnen(MessQuerschnitt mq, boolean[] einkrit, boolean[] auskrit, boolean statusUeAlt)
	{
		boolean stoerungMesswerteFs = true;
		boolean stoerungMesswerteMq = true;
		auskrit[0] = true;

		float guete = 0.0F;
    float zStau = (float)0.0;
    float zNichtStau = (float)1.0;
    float vStau = (float)0.0;
    float vNichtStau = (float)1.0;

		Staukriterium sk = new Staukriterium();
		
		if (m_parameterStauFsOrientiert == null)
		{
//			System.out.println(m_objekt.getPid() + " Noch keine Parameter erhalten. Abbruch!");
			m_debug.fine(m_objekt.getPid() + " Noch keine Parameter erhalten. Abbruch!");

			return false;
		}
		
		// Prfen, auf Deaktivierung des Staukriteriums 
		if (m_parameterStaukriterium != null)
		{
			if (m_parameterStaukriterium.getStaukriterium6().getStau().getVertrauenswrdigkeit() == 0)
			{
				System.out.println(m_objekt.getPid() + " Staukriterium 6 (Fs-Orientiert) ist deaktiviert!");
				m_debug.fine(m_objekt.getPid() + " Staukriterium 6 (Fs-Orientiert) ist deaktiviert!");
				
				return false;
			}
		}			
		
		// NUR ZUM TEST
		if (m_objekt.getPid().equals("sbaA8StgtUrsachenEinheitStauTestRA"))
		{
			System.out.println(m_objekt);
		}
		
		int N = bestimmeAnzFahrstreifenAktuellNeu(mq);
		
    System.out.println("Parameter Stau FS-Orientiert -> " + mq.getPid() +  " anzFs " + N + " " + m_parameterStauFsOrientiert);
    m_debug.fine("Parameter Stau FS-Orientiert -> " + mq.getPid() + " anzFs " + N + " " + m_parameterStauFsOrientiert);
    
		VerkehrsDatenKurzZeitTrendExtraPolationMq prognoseDaten = mq.getVerkehrsDatenKurzZeitTrendExtraPolationMq();
		System.out.println("VerkehrsDaten Prognose " + mq.getPid() + " " + prognoseDaten);
		
		if (prognoseDaten == null)
			return stoerungMesswerteMq;
		
		if (prognoseDaten.hasDatenErhalten())
		{
			stoerungMesswerteMq = false;
			
			int Q_1 = prognoseDaten.getQBP();
					
			float[] gueteFs = new float[1];
			
			int anzahlFsEin = pruefeGrenzwerteGeschwindigkeit(mq, Kriterium.SKE, gueteFs);
			int anzahlFsAus = pruefeGrenzwerteGeschwindigkeit(mq, Kriterium.SKA, gueteFs);
	
			if (gueteFs[0] > 0)
				stoerungMesswerteFs = false;
			
			guete = prognoseDaten.getQBGte() * gueteFs[0];
			
			boolean ske1 = false;
			boolean ske2 = false;
	
			boolean ska1 = false;
			boolean ska2 = false;
			boolean ska3 = false;
			
			switch (N)
			{
				case 5:
				case 4:
				case 3:
				{
					int _q_f = m_parameterStauFsOrientiert.getGrenzwerte3Bis5Fs(N).getQ_f();
					int _q_g = m_parameterStauFsOrientiert.getGrenzwerte3Bis5Fs(N).getQ_g();
					
					int _x1 = m_parameterStauFsOrientiert.getGrenzwerte3Bis5Fs(N).getX1();
					int _x2 = m_parameterStauFsOrientiert.getGrenzwerte3Bis5Fs(N).getX2();
					int _x3 = m_parameterStauFsOrientiert.getGrenzwerte3Bis5Fs(N).getX3();
	
					/**
					 * SKE1
					 */
					ske1 = ((Q_1 < _q_f) && (anzahlFsEin == N));
	
					ske2 = ((Q_1 >= _q_f) && (anzahlFsEin >= _x1));
					
					einkrit[0] = (ske1 | ske2);
					
					/**
					 * Wenn Einschaltkriterium erfllt, Ausschaltkriterium nicht mehr berechnen.
					 */
					if (einkrit[0])
					{
						auskrit[0] = false;
						break;
					}
					
					/**
					 * SKA1
					 */
					
					ska1 = ((Q_1 < _q_f) && (anzahlFsAus >= _x2));
	
					ska2 = ((Q_1 >= _q_f) && (Q_1 < _q_g) && (anzahlFsAus >= _x3));
	
					ska3 = ((Q_1 >= _q_g) && (anzahlFsAus == N));
	
					auskrit[0] = (ska1 | ska2 | ska3);
					
					break;
				}
				
				case 2:
				{
					int _q_f = m_parameterStauFsOrientiert.getGrenzwerte1Bis2Fs(N).getQ_f();
	
					/**
					 * SKE1
					 */
					ske1 = ((Q_1 < _q_f) && (anzahlFsEin == N));
	
					ske2 = ((Q_1 >= _q_f) && (anzahlFsEin >= 1));
					
					einkrit[0] = (ske1 | ske2);
	
					/**
					 * Wenn Einschaltkriterium erfllt, Ausschaltkriterium nicht mehr berechnen.
					 */
					if (einkrit[0])
					{
						auskrit[0] = false;
						break;
					}
					
					/**
					 * SKA1
					 */
					
					ska1 = (anzahlFsAus == N);
	
					auskrit[0] = ska1;
				
					break;
				}
				
				case 1:
				{
					int _q_f = m_parameterStauFsOrientiert.getGrenzwerte1Bis2Fs(N).getQ_f();
	
					/**
					 * SKE1
					 */
					ske1 = ((Q_1 < _q_f) && (anzahlFsEin == N));
					
					einkrit[0] = ske1;
	
					/**
					 * Wenn Einschaltkriterium erfllt, Ausschaltkriterium nicht mehr berechnen.
					 */
					if (einkrit[0])
					{
						auskrit[0] = false;
						break;
					}
					
					/**
					 * SKA1
					 */				
					ska1 = (anzahlFsAus == N);
	
					auskrit[0] = ska1;
				
					break;				
				}
				
				default:
				{
					einkrit[0] = false;
					auskrit[0] = true;
					
					break;
				}
			}

			System.out.println(m_objekt.getName() + " N=" + N + " QB=" + Q_1 + " AnzahlFsEin=" + anzahlFsEin + " AnzahlFsAus=" + anzahlFsAus + " ske1=" + ske1 + " ske2=" + ske2 + " einkrit=" + einkrit[0] + " ska1=" + ska1+ " ska2=" + ska2 + " ska3=" + ska3 + " auskrit=" + auskrit[0]);
		}		

		if (!stoerungMesswerteMq)
		{
			if (m_parameterStaukriterium != null)
			{
				vNichtStau = m_parameterStaukriterium.getStaukriterium6().getNichtStau().getVertrauenswrdigkeit();
				vStau = m_parameterStaukriterium.getStaukriterium6().getStau().getVertrauenswrdigkeit();
				
				zNichtStau = m_parameterStaukriterium.getStaukriterium6().getNichtStau().getZugehrigkeit();
				zStau = m_parameterStaukriterium.getStaukriterium6().getStau().getZugehrigkeit();
			}			
			
			zStau *= StaukriteriumKlassifizierung.zugehoerigkeitStau(einkrit[0], auskrit[0], m_staukriterium6ZustandVerkehrlich);
			zNichtStau *= StaukriteriumKlassifizierung.zugehoerigkeitNichtStau(einkrit[0], auskrit[0], m_staukriterium6ZustandVerkehrlich);

			m_staukriterium6ZustandVerkehrlich = bestimmeZustandVerkehrlich(einkrit[0], auskrit[0], m_staukriterium6ZustandVerkehrlich);
			boolean zustandVerkehrlich = bestimmeZustandVerkehrlich(einkrit[0], auskrit[0], m_staukriterium6ZustandVerkehrlich);

			m_staukriterium6.setPid(mq.getPid());
			m_staukriterium6.setGteNichtStau(guete);
			m_staukriterium6.setGteStau(guete);
			m_staukriterium6.setVertrauenswrdigkeitNichtStau(vNichtStau);
			m_staukriterium6.setVertrauenswrdigkeitStau(vStau);
			m_staukriterium6.setZugehrigkeitNichtStau(zNichtStau);
			m_staukriterium6.setZugehrigkeitStau(zStau);
			m_staukriterium6.setZustandVerkehrlich(zustandVerkehrlich);
			
			sk.setPid(mq.getPid());
			sk.setGteNichtStau(guete);
			sk.setGteStau(guete);
			sk.setVertrauenswrdigkeitNichtStau(vNichtStau);
			sk.setVertrauenswrdigkeitStau(vStau);
			sk.setZugehrigkeitNichtStau(zNichtStau);
			sk.setZugehrigkeitStau(zStau);
			sk.setZustandVerkehrlich(zustandVerkehrlich);
		}
		else
		{
			m_staukriterium6.setPid(mq.getPid());
			m_staukriterium6.init();
			
			sk.setPid(mq.getPid());
			sk.init();
			
			m_staukriterium6ZustandVerkehrlich = false;
		}

		if (sk.getGteNichtStau() >= m_parameterStauFsOrientiert.getGuete())
			m_staukriteriumListe.add(sk);
		else
		{
			einkrit[0] = false;
			auskrit[0] = true;

			m_staukriterium6ZustandVerkehrlich = false;
			
			System.out.println(mq.getPid() + " Staukriterium 6 Grenzwert Gte (" + m_parameterStauFsOrientiert.getGuete() + ") nicht erreicht");
		}

		System.out.println(mq.getPid() + " Staukriterium 6 -> " + sk);
							
		return stoerungMesswerteFs | stoerungMesswerteMq;
	}

	/**
	 * Prft die Fahrstreifengeschwindigkeit vgKfz mit dem entsprechenden Grenzwert auf ber-/Unterschreitung
	 * entsprechend dem gegebenen Kriterium.
	 * 
	 * @param mq MessQuerschnitt.
	 * @param kriterium Kriterium fr den Vergleich.
	 * @return Anzahl der Fahrstreifen, die das Kriterium erfllen.
	 */
	private int pruefeGrenzwerteGeschwindigkeit(MessQuerschnitt mq, Kriterium kriterium, float gueteFs[])
	{
		int anzahlFs = 0;
		
		List<SystemObject> _fahrstreifen = mq.getFahrstreifen();
		if (_fahrstreifen == null)
			return anzahlFs;

		float gueteFsSumme = 0.0F;
		int anzGueteWerte = 0;
		
		/**
		 * Loop ber alle Fahrstreifen des MQ's.
		 * Prfen der nicht gesperrten (passivierten) Fahrstreifen auf Unterschreitung der Schwellwerte.
		 */
		for (int i = 0, ispPara = 0; i < _fahrstreifen.size(); i++)
		{
			SystemObject fs = _fahrstreifen.get(i);
			
			// Fahrstreifen gesperrt ?
			if (isSpurGesperrt(mq.getObjektFahrstreifen(fs.getPid())))
			{
				System.out.println("Fahrstreifen gesperrt! " + fs);
				m_debug.fine("Fahrstreifen gesperrt! " + fs);
				
				continue;
			}
				    	    
			VerkehrsDatenKurzZeitFs datenFs = mq.getVerkehrsDatenKurzZeitFs(fs.getPid());
			if (datenFs == null)
				continue;
			
			// keine Daten vohanden ?
			if (!datenFs.hasDatenErhalten())
				continue;
						
			int _vKfzg = datenFs.getVgKfz();
			gueteFsSumme += datenFs.getVgKfzGte();
			anzGueteWerte++;
			
			int _paraV = bestVSchwellwert(ispPara, mq.getObjektFahrstreifen(fs.getPid()), kriterium);
			ispPara++;

			System.out.println(m_objekt.getName() + " pruefeGrenzwerteGeschwindigkeit -> " + fs + " SK=" + kriterium + " vKfzg=" + _vKfzg + " paraV=" + _paraV + " ispPara=" + (ispPara-1));


			if (kriterium.equals(Kriterium.SKE))
			{
				// Kein Schwellwert bestimmt, dann zurck
				if ((_paraV == 99) || (_paraV <= 0))
				{
					continue;
				}
				
				// Geswchindigkeit prfen
				if (_vKfzg < _paraV)
				{
					anzahlFs++;
				}
			}
			else
			{
				// Kein Schwellwert bestimmt, dann zurck
				if (_paraV == 99)
				{
					continue;
				}
				
				// Geswchindigkeit prfen
				if (_vKfzg > _paraV)
				{
					anzahlFs++;
				}				
			}
		}
		
		if (anzGueteWerte > 0)
			gueteFs[0] = gueteFsSumme / anzGueteWerte;
		
		System.out.println(m_objekt.getName() + " pruefeGrenzwerteGeschwindigkeit -> SK=" + kriterium + " AnzahlFs=" + anzahlFs + " GteFs=" + gueteFs[0]);
		
		return anzahlFs;
	}

	/**
	 * Bestimmt den parametrierten Schwellwert fr den Fahrstreifen anhand der aktuellen Rckmeldung
	 * der zugehrigen Anzeige. Kann keine aktuelle Rckmeldung der Anzeige bestimmt werden, wird 'Dunkel' angenommen.
	 * 
	 * @param indexPara Index Parameter fr den Fahrstreifen.
	 * @param fs Objekt FahrStreifen.
	 * @param kriterium Kennung Ein-/Ausschaltkriterium. 
	 * @return Schwellwert.
	 */
	private int bestVSchwellwert(int indexPara, FahrStreifen fs, Kriterium kriterium)
	{
		int schwellwert;
		SystemObject code = null;
		
		
		schwellwert = 99;

		Anzeige anzeige = m_hmAnzeigen.get(fs.getLageIndex());
		if (anzeige != null)
		{
			if (anzeige.getSchaltbildDaten() != null)
			{
				code = anzeige.getSchaltbildDaten().getReferenceValue("AnzeigeInhalt").getSystemObject();
			}
		}

		// Kein Code gefunden, Dunkel verwenden
		if (code == null)
		{
			code = m_objektWvzInhaltDunkel;
		}			
		
		if (kriterium.equals(Kriterium.SKE))
		{
			schwellwert = m_parameterStauFsOrientiert.getSchwellwertVein(code, indexPara);			
		}
		else if (kriterium.equals(Kriterium.SKA))
		{
			schwellwert = m_parameterStauFsOrientiert.getSchwellwertVaus(code, indexPara);						
		}
		
		System.out.println(m_objekt.getName() + " bestVSchwellwert -> Anzeige=" + ((anzeige != null) ? anzeige.getName() : "n/a") + " SK=" + kriterium + " CODE='" + code.getName() + "' indexPara=" + indexPara + " Schwellwert=" + schwellwert);
		
		return schwellwert;
	}

	/**
	 * Diese Methode berechnet vkDiff aus den bergebenen Daten.
	 * 
	 * @param prognoseDaten Prognosedaten Messquerschnitt 1.
	 * @param prognoseDaten2 Prognosedaten Messquerschnitt 2.
	 * @param parameter Parmeter.
	 * @return berechneter Wert vkDiff.
	 */
	private float berechneVkDiff(VerkehrsDatenKurzZeitTrendExtraPolationMq prognoseDaten, VerkehrsDatenKurzZeitTrendExtraPolationMq prognoseDaten2, ParameterStauStoerfallFahrstreifen parameter)
	{
		double _vKfzP1 = prognoseDaten.getVKfzP();
		double _dP1 = prognoseDaten.getKKfzP();

		double _vKfzP2 = prognoseDaten2.getVKfzP();
		double _dP2 = prognoseDaten2.getKKfzP();
		
    double _vfrei = parameter.getVFrei();
    double _dMax = parameter.getDMax();
    
    
    if ((_vKfzP1 >= 0) && (_vKfzP2 >= 0) && (_dP1 >= 0) && (_dP2 >= 0))
    {
	    if ((_vfrei > 0) && (_dMax > 0))
	    {
	//    	System.out.println("Parameter     -> vFrei " + _vfrei + " dMax " + _dMax);    	
	//    	System.out.println("Messwerte MQ1 -> vKfzP " + _vKfzP1 + " dP " + _dP1);
	//    	System.out.println("Messwerte MQ2 -> vKfzP " + _vKfzP2 + " dP " + _dP2);
	    	    	
	    	double t1 = Math.sqrt((Math.pow(((_vfrei - _vKfzP1) / _vfrei), 2) + Math.pow((_dP1 / _dMax), 2)));
	    	double t2 = Math.sqrt((Math.pow(((_vfrei - _vKfzP2) / _vfrei), 2) + Math.pow((_dP2 / _dMax), 2)));
	
	//    	System.out.println("----------------> T1 " + t1 + " T2 " + t2 + " vkDiff " + (t1-t2));
	    	
	    	return (float)(t1-t2);
	    }
    }
    
		return 0;
	}

	/**
	 * Diese Methode bestimmt die Anzahl der aktuell am Messquerschnitt vorhandenen
	 * Fahrstreifen abzglich der logisch passivierten Fahrstreifen.
	 * 
	 * @param mq Klasse Messquerschnitt.
	 * @return aktuelle Anzahl der Fahrstreifen des Messquerschnitt. 
	 */
	private int bestimmeAnzFahrstreifenAktuell(MessQuerschnitt mq)
	{
		int anzFs = mq.getAnzahlFahrstreifen();

		if (m_passivierungFahrStreifen != null)
		{
			List<SystemObject> lfs = mq.getFahrstreifen();
			for (int i = 0; i < lfs.size(); i++)
			{
				if (m_passivierungFahrStreifen.isFahrStreifenPassiviert(lfs.get(i)))
				{
					anzFs--;
				}				
			}			
		}
		
		return anzFs;
	}

	/**
	 * Diese Methode bestimmt die Anzahl der aktuell am Messquerschnitt vorhandenen
	 * Fahrstreifen abzglich der logisch passivierten Fahrstreifen und der durch
	 * Kreuz und Gelbpfeile gesperrten Spuren.
	 * 
	 * @param mq Klasse Messquerschnitt.
	 * @return aktuelle Anzahl der Fahrstreifen des Messquerschnitt. 
	 */
	private int bestimmeAnzFahrstreifenAktuellNeu(MessQuerschnitt mq)
	{
		int anzFs = mq.getAnzahlFahrstreifen();

		List<SystemObject> lfs = mq.getFahrstreifen();
		for (int i = 0; i < lfs.size(); i++)
		{
			if (isSpurGesperrt(mq.getObjektFahrstreifen(lfs.get(i).getPid())))
			{
				anzFs--;
			}			
		}
		
		return anzFs;
	}
	
	/**
	 * Prft, ob die Fahrspur des gegebenen Fahrstreifens gesperrt ist.</p>
	 * Die Fahrspur gilt als gesperrt, wenn der Fahrstreifen passiviert oder aktuell
	 * ein Kreuz oder Gelbpfeil anzgezeigt wird.
	 * 
	 * @param fahrstreifen zu prfender Fahrstreifen
	 * @return true wenn Spur gesperrt, ansonsten false.
	 */
	private boolean isSpurGesperrt(FahrStreifen fahrstreifen)
	{
		SystemObject objFs = fahrstreifen.getObjekt();
		
		if (isFahrstreifenPassiviert(objFs))
			return true;

//		Anzeige anzeige = bestimmeAnzeige(fahrstreifen.getLage());
		Anzeige anzeige = m_hmAnzeigen.get(fahrstreifen.getLageIndex());
		if (anzeige != null)
		{
			if (anzeige.getSchaltbildDaten() != null)
			{	
				SystemObject anzeigeInhalt = anzeige.getSchaltbildDaten().getReferenceValue("AnzeigeInhalt").getSystemObject();
				if (anzeigeInhalt == null)
					return false;
				
				ConfigurationObject co = (ConfigurationObject)anzeigeInhalt;
				Data data = co.getConfigurationData(m_dataModel.getAttributeGroup("atg.wvzInhalt"));
				
				String bildinhalt = data.getTextValue("Bildinhalt").getValueText();
				if (bildinhalt != null)
				{
					if ((bildinhalt.equals(Definitionen.KREUZ)) ||
							(bildinhalt.equals(Definitionen.PFEILL)) ||
							(bildinhalt.equals(Definitionen.PFEILR)))
					{
						return true;
					}
				}
			}
		}
		
		return false;
	}
	
	/**
	 * Prft, ob der bergebene Fahrstreifen logisch passiviert ist.
	 * 
	 * @param fs Systemobjekt Fahrstreifen.
	 * @return true wenn Fahrstreifen passiviert, ansonsten false;
	 */
	private boolean isFahrstreifenPassiviert(SystemObject fs)
	{
		if (m_passivierungFahrStreifen != null)
		{
			if (m_passivierungFahrStreifen.isFahrStreifenPassiviert(fs))
			{
				return true;
			}
		}
		
		return false;
	}
	/**
	 * Diese Methode bestimmt aus dem bergebenen Ein-/Ausschaltkriterium und dem aktuellen Status
	 * der Ursacheneinheit den neuen Status und gibt ihn zurck.
	 * 
	 * @param einschaltKriterium Status Einschaltkriterium.
	 * @param ausschaltKriterium Status Ausschaltkriterium.
	 * @param statusAkt Aktueller Status der Ursacheneinheit.
	 * @return true wenn Einschaltkriterium erfllt, false wenn Ausschaltkriterium erfllt, ansonsten aktueller Status.
	 */
	private boolean bestimmeKriteriumMq(boolean einschaltKriterium, boolean ausschaltKriterium, boolean statusAkt)
	{
		boolean zurueck = statusAkt;

		// Neuer Status bestimmen
    if (!statusAkt) // UE nicht aktiv
    {
    	if (einschaltKriterium)
    		zurueck = true;
    }
    else
    {
    	if (ausschaltKriterium)
    		zurueck = false;
    }
    
		return zurueck;
	}
	
	/**
	 * 
	 * @param einschaltKriterium
	 * @param ausschaltKriterium
	 * @param zustandAkt
	 * @return
	 */
	private boolean bestimmeZustandVerkehrlich(boolean einschaltKriterium, boolean ausschaltKriterium, boolean zustandAkt)
	{
		boolean zurueck = zustandAkt;
		
		if (!zustandAkt) // Staukriterium nicht aktiv
		{
			if (einschaltKriterium)
			{
				zurueck = true;
			}
		}
		else // Staukriterium aktiv
		{
			if (ausschaltKriterium)
			{
				zurueck = false;
			}
		}

		return zurueck;
	}

	/**
	 * Diese Methode versendet die Analysedaten der Situationsbestimmung ber den Datenverteiler.
	 * 
	 * @param analyse Klasse mit den Analysedaten.
	 */
	private void sendeAnalyseDaten(SituationAnalyseMq analyse)
	{
		DataDescription dd = new DataDescription(m_atgAnalyse, m_aspAnalyse);

		Data data_neu = m_connection.createData(m_atgAnalyse);
		if (data_neu != null)
		{
			data_neu.getScaledValue("Einschaltkriterium").setText((analyse.isEinschaltkriterium())? "Ja" : "Nein"); 
			data_neu.getScaledValue("Ausschaltkriterium").setText((analyse.isAusschaltkriterium())? "Ja" : "Nein"); 
			
			data_neu.getArray("Status").setLength(0);

			for (int i = 0; i < analyse.getLength(); i++)
			{
				data_neu.getArray("Status").setLength(i+1);
				
				Data element = data_neu.getArray("Status").getItem(i);
				if (element != null)
				{
					element.getReferenceValue("MessQuerschnitt").setSystemObject(analyse.getItem(i).getMessQuerschnitt());				
					element.getUnscaledValue("Aktiv").setText(analyse.getItem(i).getAktiv());
				}			
			}
		}

		ResultData erg = new ResultData(m_objekt, dd, m_connection.getTime(), data_neu);
		
		try
		{
			m_connection.sendData(erg);
		}
		catch (DataNotSubscribedException e)
		{
			e.printStackTrace();
		}
		catch (SendSubscriptionNotConfirmed e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Diese Klasse versendet die Daten der Zusammenfassung der Staukriterien ber den Datenverteiler.
	 * 
	 * @param zusammenfassung Klasse mit den Daten der Zusammenfassung.
	 */
	private void sendeStaukriteriumZusammenfassung(StaukriteriumZusammenfassung zusammenfassung)
	{
		if (zusammenfassung != null)
		{
			DataDescription dd = new DataDescription(m_atgZusammenfassungStaukriterien, m_aspAnalyse);

			Data data_neu = m_connection.createData(m_atgZusammenfassungStaukriterien);
			if (data_neu != null)
			{
				data_neu.getScaledValue("Staukriterium").setText(zusammenfassung.getStaukriterium());
								
			  if (zusammenfassung.getNichtStau().getP() >= 1)
					data_neu.getItem("Nicht-Stau").getScaledValue("P").set(1);
			  else if ((zusammenfassung.getNichtStau().getP() > 0) && (zusammenfassung.getNichtStau().getP() <= 1))
					data_neu.getItem("Nicht-Stau").getScaledValue("P").set(zusammenfassung.getNichtStau().getP());
				else					
					data_neu.getItem("Nicht-Stau").getScaledValue("P").set(0);
					
				if ((zusammenfassung.getNichtStau().getPn() > 0) && (zusammenfassung.getNichtStau().getPn() <= 1))
					data_neu.getItem("Nicht-Stau").getScaledValue("Pn").set(zusammenfassung.getNichtStau().getPn());
				else
					data_neu.getItem("Nicht-Stau").getScaledValue("Pn").set(0);
					
				if ((zusammenfassung.getNichtStau().getG() > 0) && (zusammenfassung.getNichtStau().getG() <= 1))
					data_neu.getItem("Nicht-Stau").getScaledValue("G").set(zusammenfassung.getNichtStau().getG());
				else
					data_neu.getItem("Nicht-Stau").getScaledValue("G").set(0);
					
			  if (zusammenfassung.getStau().getP() >= 1)
					data_neu.getItem("Stau").getScaledValue("P").set(1);
			  else if ((zusammenfassung.getStau().getP() > 0) && (zusammenfassung.getStau().getP() <= 1))
					data_neu.getItem("Stau").getScaledValue("P").set(zusammenfassung.getStau().getP());
				else					
					data_neu.getItem("Stau").getScaledValue("P").set(0);
			  
				if ((zusammenfassung.getStau().getPn() > 0) && (zusammenfassung.getStau().getPn() <= 1))
					data_neu.getItem("Stau").getScaledValue("Pn").set(zusammenfassung.getStau().getPn());
				else
					data_neu.getItem("Stau").getScaledValue("Pn").set(0);

				if ((zusammenfassung.getStau().getG() > 0) && (zusammenfassung.getStau().getG() <= 1))
					data_neu.getItem("Stau").getScaledValue("G").set(zusammenfassung.getStau().getG());
				else
					data_neu.getItem("Stau").getScaledValue("G").set(0);
			}

			ResultData erg = new ResultData(m_objekt, dd, m_connection.getTime(), data_neu);
			
			try
			{
				m_connection.sendData(erg);
			}
			catch (DataNotSubscribedException e)
			{
				e.printStackTrace();
			}
			catch (SendSubscriptionNotConfirmed e)
			{
				e.printStackTrace();
			}

		}
	}
	/**
	 * Diese Klasse versendet die Daten der Ergebnisse der Staukriterien ber den Datenverteiler.
	 */
	private void sendeStaukriteriumErgebnis()
	{
		float vIni = 0.0F;
		float zIni = 0.0F;
		float gIni = 0.0F;
		
		DataDescription dd = new DataDescription(m_atgErgebnisStaukriterien, m_aspAnalyse);

		Data data_neu = m_connection.createData(m_atgErgebnisStaukriterien);
		if (data_neu != null)
		{
			data_neu.getItem("Staukriterium1").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
			data_neu.getItem("Staukriterium1").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(zIni);
			data_neu.getItem("Staukriterium1").getItem("Nicht-Stau").getScaledValue("Gte").set(gIni);
			data_neu.getItem("Staukriterium1").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
			data_neu.getItem("Staukriterium1").getItem("Stau").getScaledValue("Zugehrigkeit").set(zIni);
			data_neu.getItem("Staukriterium1").getItem("Stau").getScaledValue("Gte").set(gIni);					

			data_neu.getItem("Staukriterium2").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
			data_neu.getItem("Staukriterium2").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(zIni);
			data_neu.getItem("Staukriterium2").getItem("Nicht-Stau").getScaledValue("Gte").set(gIni);
			data_neu.getItem("Staukriterium2").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
			data_neu.getItem("Staukriterium2").getItem("Stau").getScaledValue("Zugehrigkeit").set(zIni);
			data_neu.getItem("Staukriterium2").getItem("Stau").getScaledValue("Gte").set(gIni);					

			data_neu.getItem("Staukriterium3").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
			data_neu.getItem("Staukriterium3").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(zIni);
			data_neu.getItem("Staukriterium3").getItem("Nicht-Stau").getScaledValue("Gte").set(gIni);
			data_neu.getItem("Staukriterium3").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
			data_neu.getItem("Staukriterium3").getItem("Stau").getScaledValue("Zugehrigkeit").set(zIni);
			data_neu.getItem("Staukriterium3").getItem("Stau").getScaledValue("Gte").set(gIni);					

			data_neu.getItem("Staukriterium4").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
			data_neu.getItem("Staukriterium4").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(zIni);
			data_neu.getItem("Staukriterium4").getItem("Nicht-Stau").getScaledValue("Gte").set(gIni);
			data_neu.getItem("Staukriterium4").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
			data_neu.getItem("Staukriterium4").getItem("Stau").getScaledValue("Zugehrigkeit").set(zIni);
			data_neu.getItem("Staukriterium4").getItem("Stau").getScaledValue("Gte").set(gIni);					

			data_neu.getItem("Staukriterium5").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
			data_neu.getItem("Staukriterium5").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(zIni);
			data_neu.getItem("Staukriterium5").getItem("Nicht-Stau").getScaledValue("Gte").set(gIni);
			data_neu.getItem("Staukriterium5").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
			data_neu.getItem("Staukriterium5").getItem("Stau").getScaledValue("Zugehrigkeit").set(zIni);
			data_neu.getItem("Staukriterium5").getItem("Stau").getScaledValue("Gte").set(gIni);					

			if (m_modellStauFsOrientiertvorhanden)
			{
				data_neu.getItem("Staukriterium6").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
				data_neu.getItem("Staukriterium6").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(zIni);
				data_neu.getItem("Staukriterium6").getItem("Nicht-Stau").getScaledValue("Gte").set(gIni);
				data_neu.getItem("Staukriterium6").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(vIni);
				data_neu.getItem("Staukriterium6").getItem("Stau").getScaledValue("Zugehrigkeit").set(zIni);
				data_neu.getItem("Staukriterium6").getItem("Stau").getScaledValue("Gte").set(gIni);									
			}
			
			if (m_staukriterium1 != null)
			{
//				if ((m_staukriterium1.getVertrauenswrdigkeitNichtStau() > 0) && (m_staukriterium1.getVertrauenswrdigkeitNichtStau() <= 1))
//					data_neu.getItem("Staukriterium1").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium1.getVertrauenswrdigkeitNichtStau());
//					
//				if ((m_staukriterium1.getZugehrigkeitNichtStau() > 0) && (m_staukriterium1.getZugehrigkeitNichtStau() <= 1))
//					data_neu.getItem("Staukriterium1").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium1.getZugehrigkeitNichtStau());
//				
//				if ((m_staukriterium1.getGteNichtStau() > 0) && (m_staukriterium1.getGteNichtStau() <= 1))
//					data_neu.getItem("Staukriterium1").getItem("Nicht-Stau").getScaledValue("Gte").set(m_staukriterium1.getGteNichtStau());
//				
//				if ((m_staukriterium1.getVertrauenswrdigkeitStau() > 0) && (m_staukriterium1.getVertrauenswrdigkeitStau() <= 1))
//					data_neu.getItem("Staukriterium1").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium1.getVertrauenswrdigkeitStau());
//				
//				if ((m_staukriterium1.getZugehrigkeitStau() > 0) && (m_staukriterium1.getZugehrigkeitStau() <= 1))
//					data_neu.getItem("Staukriterium1").getItem("Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium1.getZugehrigkeitStau());
//				
//				if ((m_staukriterium1.getGteStau() > 0) && (m_staukriterium1.getGteStau() <= 1))
//					data_neu.getItem("Staukriterium1").getItem("Stau").getScaledValue("Gte").set(m_staukriterium1.getGteStau());

				if ((m_staukriterium1.getVertrauenswrdigkeitNichtStau() >= 0) && (m_staukriterium1.getVertrauenswrdigkeitNichtStau() <= 1))
					data_neu.getItem("Staukriterium1").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium1.getVertrauenswrdigkeitNichtStau());
					
				if ((m_staukriterium1.getZugehrigkeitNichtStau() >= 0) && (m_staukriterium1.getZugehrigkeitNichtStau() <= 1))
					data_neu.getItem("Staukriterium1").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium1.getZugehrigkeitNichtStau());
				
				if ((m_staukriterium1.getGteNichtStau() >= 0) && (m_staukriterium1.getGteNichtStau() <= 1))
					data_neu.getItem("Staukriterium1").getItem("Nicht-Stau").getScaledValue("Gte").set(m_staukriterium1.getGteNichtStau());
				
				if ((m_staukriterium1.getVertrauenswrdigkeitStau() >= 0) && (m_staukriterium1.getVertrauenswrdigkeitStau() <= 1))
					data_neu.getItem("Staukriterium1").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium1.getVertrauenswrdigkeitStau());
				
				if ((m_staukriterium1.getZugehrigkeitStau() >= 0) && (m_staukriterium1.getZugehrigkeitStau() <= 1))
					data_neu.getItem("Staukriterium1").getItem("Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium1.getZugehrigkeitStau());
				
				if ((m_staukriterium1.getGteStau() >= 0) && (m_staukriterium1.getGteStau() <= 1))
					data_neu.getItem("Staukriterium1").getItem("Stau").getScaledValue("Gte").set(m_staukriterium1.getGteStau());

			}

			if (m_staukriterium2 != null)
			{
				if ((m_staukriterium2.getVertrauenswrdigkeitNichtStau() > 0) && (m_staukriterium2.getVertrauenswrdigkeitNichtStau() <= 1))
					data_neu.getItem("Staukriterium2").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium2.getVertrauenswrdigkeitNichtStau());
					
				if ((m_staukriterium2.getZugehrigkeitNichtStau() > 0) && (m_staukriterium2.getZugehrigkeitNichtStau() <= 1))
					data_neu.getItem("Staukriterium2").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium2.getZugehrigkeitNichtStau());
				
				if ((m_staukriterium2.getGteNichtStau() > 0) && (m_staukriterium2.getGteNichtStau() <= 1))
					data_neu.getItem("Staukriterium2").getItem("Nicht-Stau").getScaledValue("Gte").set(m_staukriterium2.getGteNichtStau());
				
				if ((m_staukriterium2.getVertrauenswrdigkeitStau() > 0) && (m_staukriterium2.getVertrauenswrdigkeitStau() <= 1))
					data_neu.getItem("Staukriterium2").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium2.getVertrauenswrdigkeitStau());
				
				if ((m_staukriterium2.getZugehrigkeitStau() > 0) && (m_staukriterium2.getZugehrigkeitStau() <= 1))
					data_neu.getItem("Staukriterium2").getItem("Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium2.getZugehrigkeitStau());
				
				if ((m_staukriterium2.getGteStau() > 0) && (m_staukriterium2.getGteStau() <= 1))
					data_neu.getItem("Staukriterium2").getItem("Stau").getScaledValue("Gte").set(m_staukriterium2.getGteStau());
			}

			if (m_staukriterium3 != null)
			{
				if ((m_staukriterium3.getVertrauenswrdigkeitNichtStau() > 0) && (m_staukriterium3.getVertrauenswrdigkeitNichtStau() <= 1))
					data_neu.getItem("Staukriterium3").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium3.getVertrauenswrdigkeitNichtStau());
					
				if ((m_staukriterium3.getZugehrigkeitNichtStau() > 0) && (m_staukriterium3.getZugehrigkeitNichtStau() <= 1))
					data_neu.getItem("Staukriterium3").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium3.getZugehrigkeitNichtStau());
				
				if ((m_staukriterium3.getGteNichtStau() > 0) && (m_staukriterium3.getGteNichtStau() <= 1))
					data_neu.getItem("Staukriterium3").getItem("Nicht-Stau").getScaledValue("Gte").set(m_staukriterium3.getGteNichtStau());
				
				if ((m_staukriterium3.getVertrauenswrdigkeitStau() > 0) && (m_staukriterium3.getVertrauenswrdigkeitStau() <= 1))
					data_neu.getItem("Staukriterium3").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium3.getVertrauenswrdigkeitStau());
				
				if ((m_staukriterium3.getZugehrigkeitStau() > 0) && (m_staukriterium3.getZugehrigkeitStau() <= 1))
					data_neu.getItem("Staukriterium3").getItem("Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium3.getZugehrigkeitStau());
				
				if ((m_staukriterium3.getGteStau() > 0) && (m_staukriterium3.getGteStau() <= 1))
					data_neu.getItem("Staukriterium3").getItem("Stau").getScaledValue("Gte").set(m_staukriterium3.getGteStau());
			}

			if (m_staukriterium4 != null)
			{
				if ((m_staukriterium4.getVertrauenswrdigkeitNichtStau() > 0) && (m_staukriterium4.getVertrauenswrdigkeitNichtStau() <= 1))
					data_neu.getItem("Staukriterium4").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium4.getVertrauenswrdigkeitNichtStau());
					
				if ((m_staukriterium4.getZugehrigkeitNichtStau() > 0) && (m_staukriterium4.getZugehrigkeitNichtStau() <= 1))
					data_neu.getItem("Staukriterium4").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium4.getZugehrigkeitNichtStau());
				
				if ((m_staukriterium4.getGteNichtStau() > 0) && (m_staukriterium4.getGteNichtStau() <= 1))
					data_neu.getItem("Staukriterium4").getItem("Nicht-Stau").getScaledValue("Gte").set(m_staukriterium4.getGteNichtStau());
				
				if ((m_staukriterium4.getVertrauenswrdigkeitStau() > 0) && (m_staukriterium4.getVertrauenswrdigkeitStau() <= 1))
					data_neu.getItem("Staukriterium4").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium4.getVertrauenswrdigkeitStau());
				
				if ((m_staukriterium4.getZugehrigkeitStau() > 0) && (m_staukriterium4.getZugehrigkeitStau() <= 1))
					data_neu.getItem("Staukriterium4").getItem("Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium4.getZugehrigkeitStau());
				
				if ((m_staukriterium4.getGteStau() > 0) && (m_staukriterium4.getGteStau() <= 1))
					data_neu.getItem("Staukriterium4").getItem("Stau").getScaledValue("Gte").set(m_staukriterium4.getGteStau());
			}

//			if (m_staukriterium5 != null)
//			{
//				if ((m_staukriterium5.getVertrauenswrdigkeitNichtStau() > 0) && (m_staukriterium5.getVertrauenswrdigkeitNichtStau() <= 1))
//					data_neu.getItem("Staukriterium5").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium5.getVertrauenswrdigkeitNichtStau());
//					
//				if ((m_staukriterium5.getZugehrigkeitNichtStau() > 0) && (m_staukriterium5.getZugehrigkeitNichtStau() <= 1))
//					data_neu.getItem("Staukriterium5").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium5.getZugehrigkeitNichtStau());
//				
//				if ((m_staukriterium5.getGteNichtStau() > 0) && (m_staukriterium5.getGteNichtStau() <= 1))
//					data_neu.getItem("Staukriterium5").getItem("Nicht-Stau").getScaledValue("Gte").set(m_staukriterium5.getGteNichtStau());
//				
//				if ((m_staukriterium5.getVertrauenswrdigkeitStau() > 0) && (m_staukriterium5.getVertrauenswrdigkeitStau() <= 1))
//					data_neu.getItem("Staukriterium5").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium5.getVertrauenswrdigkeitStau());
//				
//				if ((m_staukriterium5.getZugehrigkeitStau() > 0) && (m_staukriterium5.getZugehrigkeitStau() <= 1))
//					data_neu.getItem("Staukriterium5").getItem("Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium5.getZugehrigkeitStau());
//				
//				if ((m_staukriterium5.getGteStau() > 0) && (m_staukriterium5.getGteStau() <= 1))
//					data_neu.getItem("Staukriterium5").getItem("Stau").getScaledValue("Gte").set(m_staukriterium5.getGteStau());
//			}

			
			if (m_modellStauFsOrientiertvorhanden)
			{
				if (m_staukriterium6 != null)
				{
					if ((m_staukriterium6.getVertrauenswrdigkeitNichtStau() > 0) && (m_staukriterium6.getVertrauenswrdigkeitNichtStau() <= 1))
						data_neu.getItem("Staukriterium6").getItem("Nicht-Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium6.getVertrauenswrdigkeitNichtStau());
						
					if ((m_staukriterium6.getZugehrigkeitNichtStau() > 0) && (m_staukriterium6.getZugehrigkeitNichtStau() <= 1))
						data_neu.getItem("Staukriterium6").getItem("Nicht-Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium6.getZugehrigkeitNichtStau());
					
					if ((m_staukriterium6.getGteNichtStau() > 0) && (m_staukriterium6.getGteNichtStau() <= 1))
						data_neu.getItem("Staukriterium6").getItem("Nicht-Stau").getScaledValue("Gte").set(m_staukriterium6.getGteNichtStau());
					
					if ((m_staukriterium6.getVertrauenswrdigkeitStau() > 0) && (m_staukriterium6.getVertrauenswrdigkeitStau() <= 1))
						data_neu.getItem("Staukriterium6").getItem("Stau").getScaledValue("Vertrauenswrdigkeit").set(m_staukriterium6.getVertrauenswrdigkeitStau());
					
					if ((m_staukriterium6.getZugehrigkeitStau() > 0) && (m_staukriterium6.getZugehrigkeitStau() <= 1))
						data_neu.getItem("Staukriterium6").getItem("Stau").getScaledValue("Zugehrigkeit").set(m_staukriterium6.getZugehrigkeitStau());
					
					if ((m_staukriterium6.getGteStau() > 0) && (m_staukriterium6.getGteStau() <= 1))
						data_neu.getItem("Staukriterium6").getItem("Stau").getScaledValue("Gte").set(m_staukriterium6.getGteStau());
				}				
			}
			
			ResultData erg = new ResultData(m_objekt, dd, m_connection.getTime(), data_neu);
			
			try
			{
				m_connection.sendData(erg);
			}
			catch (DataNotSubscribedException e)
			{
				e.printStackTrace();
			}
			catch (SendSubscriptionNotConfirmed e)
			{
				e.printStackTrace();
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	public void update(ResultData results[])
	{		
		for (int i = 0; i < results.length; i++)
		{
			ResultData resultData = results[i];
			
    	// Datensatz Parameter empfangen
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameterStauBeleg))
    	{ 
//    		System.out.println("-------> " + resultData);
    		
    		if (m_parameterStauBeleg == null)
    		{
    			m_parameterStauBeleg = new ParameterStauBeleg(resultData);
    		}
    		else
    		{
    			m_parameterStauBeleg.setdaten(resultData);
    		}
     	}

    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameterStauGeschw))
    	{ 
//    		System.out.println("-------> " + resultData);
    		
    		if (m_parameterStauGeschw == null)
    		{
    			m_parameterStauGeschw = new ParameterStauGeschw(resultData);
    		}
    		else
    		{
    			m_parameterStauGeschw.setdaten(resultData);
    		}
     	}

    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameterStauStoerfall))
    	{ 
//    		System.out.println("-------> " + resultData);
    		
    		if (m_parameterStauStoerfall == null)
    		{
    			m_parameterStauStoerfall = new ParameterStauStoerfall(resultData);
    		}
    		else
    		{
    			m_parameterStauStoerfall.setdaten(resultData);
    		}
    		
    		erzeugeObjekteMessQuerschnittAbschnitt();
     	}

    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameterStaukriterium))
    	{ 
//    		System.out.println("-------> " + resultData);
    		
    		if (m_parameterStaukriterium == null)
    		{
    			m_parameterStaukriterium = new ParameterStaukriterium(resultData, m_modellStauFsOrientiertvorhanden);
    		}
    		else
    		{
    			m_parameterStaukriterium.setdaten(resultData);
    		}
     	}

    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameterMqZuordnung))
    	{ 
//    		System.out.println("-------> " + resultData);
    		
    		if (m_parameterMqZuordnung == null)
    		{
    			m_parameterMqZuordnung = new ParameterMqZuordnung(resultData);
    		}
    		else
    		{
    			m_mqZuordnungAlt.clear();
    			m_mqZuordnungAlt.addAll(m_parameterMqZuordnung.getMessQuerschnitte());

    			m_parameterMqZuordnung.setdaten(resultData);
    		}
     	
    		erzeugeObjekteMessQuerschnitt();
    	}

    	if (m_modellStauFsOrientiertvorhanden)
    	{
	    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameterFsOrientiert))
	    	{ 
	//    		System.out.println("-------> " + resultData);
	    		
	    		if (m_parameterStauFsOrientiert == null)
	    		{
	    			m_parameterStauFsOrientiert = new ParameterStauFsOrientiert(resultData);
	    		}
	    		else
	    		{
	    			m_parameterStauFsOrientiert.setdaten(resultData);
	    		}
	     	
	    		System.out.println(m_parameterStauFsOrientiert);
	    		
	    		erzeugeObjekteAnzeige();
	    	}
    	}
		}
	}

	/**
	 * bernahme der bergebenen Daten bzgl. der Passivierten Fahrstreifen in die Klassenvariable <code>m_passivierungFahrStreifen</code>.
	 *  
	 * @param rdata Datensatz <code>atg.sswSbaPassivierungFahrStreifen</code>
	 */
	public void updatePassivierungFahrstreifen(ResultData rdata)
	{
		if (m_passivierungFahrStreifen == null)
			m_passivierungFahrStreifen = new PassivierungFahrstreifen(rdata);
		else
			m_passivierungFahrStreifen.setdaten(rdata);
		
//		System.out.println(m_objekt.getPid() + " PassivierungFahrstreien " + m_passivierungFahrStreifen);
	}

	/**
	 * Diese Methode erzeugt aus der Liste der zugeordenten Messquerschnitte die Objekte
	 * fr die Verwaltung der Onlinedaten.
	 */
	private void erzeugeObjekteMessQuerschnitt()
	{
		if (m_parameterMqZuordnung != null)
		{
			List<SystemObject> listeMq = m_parameterMqZuordnung.getMessQuerschnitte();
			if (listeMq != null)
			{
				for (int i = 0; i < listeMq.size(); i++)
				{
					SystemObject obj = listeMq.get(i);
											
					// virtuelle MQ haben keine FahrStreifen !
					
					if (obj.getType().equals(m_connection.getDataModel().getType("typ.messQuerschnittVirtuell")))
					{
						continue;
					}
					
					if (!m_mapMessQuerschnitte.containsKey(obj.getPid()))
					{
						MessQuerschnitt mq = new MessQuerschnitt(m_connection, obj);
						if (mq != null)
						{
							mq.datenDavAnmelden();
							m_mapMessQuerschnitte.put(obj.getPid(), mq);
						}
					}
				}
			}

			// Nicht mehr bentigte Messquerschnitte aus Liste entfernen
			for (int i = 0; i < m_mqZuordnungAlt.size(); i++)
			{
				SystemObject mqAlt = m_mqZuordnungAlt.get(i);
				if (!listeMq.contains(mqAlt))
				{
					if (m_mapMessQuerschnitte.containsKey(mqAlt.getPid()))
					{
						MessQuerschnitt _mq = m_mapMessQuerschnitte.get(mqAlt.getPid());
						if (_mq != null)
						{
							System.out.println(mqAlt.getPid() + " Daten Abmelden und aus Liste lschen");
							
							_mq.datenDavAbmelden();
							m_mapMessQuerschnitte.remove(mqAlt.getPid());
						}
					}
				}
			}
		}
	}

	/**
	 * Diese Methode erzeugt aus der Liste der zugeordenten Messquerschnitte die Objekte
	 * fr die Verwaltung der Onlinedaten.
	 */
	private void erzeugeObjekteMessQuerschnittAbschnitt()
	{
		if (m_parameterStauStoerfall != null)
		{
			List<SystemObject> listeMq = m_parameterStauStoerfall.getListeMessQuerschnittAusfahrend();
			if (listeMq != null)
			{
				for (int i = 0; i < listeMq.size(); i++)
				{
					SystemObject obj = listeMq.get(i);
											
					if (!m_mapMessQuerschnitteAbschnitt.containsKey(obj.getPid()))
					{
						MessQuerschnitt mq = new MessQuerschnitt(m_connection, obj);
						if (mq != null)
						{
							mq.datenDavAnmelden();
							m_mapMessQuerschnitteAbschnitt.put(obj.getPid(), mq);
						}
					}
				}
			}

//			// Nicht mehr bentigte Messquerschnitte aus Liste entfernen
//			for (int i = 0; i < m_mqZuordnungAlt.size(); i++)
//			{
//				SystemObject mqAlt = m_mqZuordnungAlt.get(i);
//				if (!listeMq.contains(mqAlt))
//				{
//					if (m_mapMessQuerschnitte.containsKey(mqAlt.getPid()))
//					{
//						MessQuerschnitt _mq = m_mapMessQuerschnitte.get(mqAlt.getPid());
//						if (_mq != null)
//						{
//							System.out.println(mqAlt.getPid() + " Daten Abmelden und aus Liste lschen");
//							
//							_mq.datenDavAbmelden();
//							m_mapMessQuerschnitte.remove(mqAlt.getPid());
//						}
//					}
//				}
//			}
		}
	}
	
	/**
	 * Diese Methode erzeugt die Objekte fr die Verwaltung des realen und des SBA Anzeigequerschnitts
	 * entsprechend dem empfangenen Parameter. 
	 */
	private void erzeugeObjekteAnzeige()
	{
		synchronized (m_hmAnzeigen)
		{
			m_hmAnzeigen.clear();
			
			if (m_parameterStauFsOrientiert != null)
			{
				List<SystemObject> anzeigen = m_parameterStauFsOrientiert.getAnzeige();
				if (anzeigen != null)
				{
					for (int i = 0; i < anzeigen.size(); i++)
					{
						SystemObject _anzeige = anzeigen.get(i);
						Anzeige anzeige = new Anzeige(_anzeige, (ClientDavConnection)m_connection);
						
						anzeige.datenDavAnmelden();
						
						m_hmAnzeigen.put(new Integer(i), anzeige);
					}
				}				
			}
		}
	}
	
	/**
	 * Diese Methode bestimmt ein Objekt vom Typ typ.wvzInhalt aus der aktuellen konfiguration 
	 * mit dem Bildinhalt 'Dunkel' und gibt das gefundene Objekt zurck.
	 * 
	 * @return gefundenes Objekt WvzInhalt oder <code>null</code>.
	 */
	private SystemObject bestimmeWvzInhaltDunkel()
	{
		List<SystemObject> wvzInhalte = m_dataModel.getType("typ.wvzInhalt").getElements();
		if (wvzInhalte != null)
		{
			for (SystemObject wvzinhalt : wvzInhalte)
			{
//				System.out.println(wvzinhalt);
				
				ConfigurationObject co = (ConfigurationObject)wvzinhalt;
				
				Data data = co.getConfigurationData(m_dataModel.getAttributeGroup("atg.wvzInhalt"));
				if (data != null)
				{
					String bildInhalt = data.getTextValue("Bildinhalt").getValueText();
					if (bildInhalt.equals(Definitionen.DUNKEL))
					{
						return wvzinhalt;
					}
				}
			}
		}
		
		return null;
	}
	
	
}
